@@ -416,6 +416,7 @@ void PhysicalKeyboardHandler::handleEvent(StellaKey key, StellaMod mod, bool pre
   {
     case EventHandlerState::EMULATION:
     case EventHandlerState::PAUSE:
+    case EventHandlerState::PLAYBACK:
       myHandler.handleEvent(myKeyMap.get(EventMode::kEmulationMode, key, mod), pressed, repeated);
       break;
 
@@ -469,6 +470,7 @@ PhysicalKeyboardHandler::EventMappingArray PhysicalKeyboardHandler::DefaultCommo
   {Event::VCenterIncrease,          KBDK_PAGEDOWN, MOD3},
   {Event::VSizeAdjustDecrease,      KBDK_PAGEDOWN, KBDM_SHIFT | MOD3},
   {Event::VSizeAdjustIncrease,      KBDK_PAGEUP, KBDM_SHIFT | MOD3},
+  {Event::ToggleCorrectAspectRatio, KBDK_C, KBDM_CTRL},
   {Event::VolumeDecrease,           KBDK_LEFTBRACKET, MOD3},
   {Event::VolumeIncrease,           KBDK_RIGHTBRACKET, MOD3},
   {Event::SoundToggle,              KBDK_RIGHTBRACKET, KBDM_CTRL},
@@ -505,14 +507,27 @@ PhysicalKeyboardHandler::EventMappingArray PhysicalKeyboardHandler::DefaultCommo
 #ifndef BSPF_MACOS
   {Event::PreviousSetting,          KBDK_END},
   {Event::NextSetting,              KBDK_HOME},
+  {Event::PreviousSettingGroup,     KBDK_END, KBDM_CTRL},
+  {Event::NextSettingGroup,         KBDK_HOME, KBDM_CTRL},
 #else
+  // HOME & END keys are swapped on Mac keyboards
   {Event::PreviousSetting,          KBDK_HOME},
   {Event::NextSetting,              KBDK_END},
+  {Event::PreviousSettingGroup,     KBDK_HOME, KBDM_CTRL},
+  {Event::NextSettingGroup,         KBDK_END, KBDM_CTRL},
 #endif
+  {Event::PreviousSetting,          KBDK_KP_1},
+  {Event::NextSetting,              KBDK_KP_7},
+  {Event::PreviousSettingGroup,     KBDK_KP_1, KBDM_CTRL},
+  {Event::NextSettingGroup,         KBDK_KP_7, KBDM_CTRL},
   {Event::SettingDecrease,          KBDK_PAGEDOWN},
+  {Event::SettingDecrease,          KBDK_KP_3, KBDM_CTRL},
   {Event::SettingIncrease,          KBDK_PAGEUP},
+  {Event::SettingIncrease,          KBDK_KP_9, KBDM_CTRL},
 
   {Event::ToggleInter,              KBDK_I, KBDM_CTRL},
+  {Event::DecreaseSpeed,            KBDK_S, KBDM_SHIFT | KBDM_CTRL},
+  {Event::IncreaseSpeed,            KBDK_S, KBDM_CTRL },
   {Event::ToggleTurbo,              KBDK_T, KBDM_CTRL},
   {Event::ToggleJitter,             KBDK_J, MOD3},
   {Event::ToggleFrameStats,         KBDK_L, MOD3},
@@ -522,6 +537,9 @@ PhysicalKeyboardHandler::EventMappingArray PhysicalKeyboardHandler::DefaultCommo
   {Event::ToggleContSnapshots,      KBDK_S, MOD3},
   {Event::ToggleContSnapshotsFrame, KBDK_S, KBDM_SHIFT | MOD3},
 #endif
+
+  {Event::DecreaseAutoFire,         KBDK_A, KBDM_SHIFT | KBDM_CTRL},
+  {Event::IncreaseAutoFire,         KBDK_A, KBDM_CTRL },
   {Event::HandleMouseControl,       KBDK_0, KBDM_CTRL},
   {Event::ToggleGrabMouse,          KBDK_G, KBDM_CTRL},
   {Event::ToggleSAPortOrder,        KBDK_1, KBDM_CTRL},
@@ -552,6 +570,7 @@ PhysicalKeyboardHandler::EventMappingArray PhysicalKeyboardHandler::DefaultCommo
   {Event::Unwind1Menu,              KBDK_RIGHT, MOD3},
   {Event::Unwind10Menu,             KBDK_RIGHT, KBDM_SHIFT | MOD3},
   {Event::UnwindAllMenu,            KBDK_UP, MOD3},
+  {Event::TogglePlayBackMode,       KBDK_SPACE, KBDM_SHIFT},
 
 #if defined(RETRON77)
   {Event::ConsoleColorToggle,       KBDK_F4},         // back ("COLOR","B/W")
@@ -640,6 +659,7 @@ PhysicalKeyboardHandler::EventMappingArray PhysicalKeyboardHandler::DefaultJoyst
   {Event::JoystickZeroFire9,        KBDK_5},
   {Event::JoystickZeroFire9,        KBDK_RCTRL},
   {Event::JoystickZeroFire9,        KBDK_KP_3},
+
   {Event::JoystickOneUp,            KBDK_Y},
   {Event::JoystickOneDown,          KBDK_H},
   {Event::JoystickOneLeft,          KBDK_G},
@@ -647,6 +667,22 @@ PhysicalKeyboardHandler::EventMappingArray PhysicalKeyboardHandler::DefaultJoyst
   {Event::JoystickOneFire,          KBDK_F},
   {Event::JoystickOneFire5,         KBDK_6},
   {Event::JoystickOneFire9,         KBDK_7},
+
+  {Event::JoystickTwoUp,            KBDK_UP, KBDM_SHIFT},
+  {Event::JoystickTwoDown,          KBDK_DOWN, KBDM_SHIFT},
+  {Event::JoystickTwoLeft,          KBDK_LEFT, KBDM_SHIFT},
+  {Event::JoystickTwoRight,         KBDK_RIGHT, KBDM_SHIFT},
+  {Event::JoystickTwoUp,            KBDK_KP_8, KBDM_SHIFT},
+  {Event::JoystickTwoDown,          KBDK_KP_2, KBDM_SHIFT},
+  {Event::JoystickTwoLeft,          KBDK_KP_4, KBDM_SHIFT},
+  {Event::JoystickTwoRight,         KBDK_KP_6, KBDM_SHIFT},
+  {Event::JoystickTwoFire,          KBDK_SPACE, KBDM_SHIFT},
+
+  {Event::JoystickThreeUp,          KBDK_Y, KBDM_SHIFT},
+  {Event::JoystickThreeDown,        KBDK_H, KBDM_SHIFT},
+  {Event::JoystickThreeLeft,        KBDK_G, KBDM_SHIFT},
+  {Event::JoystickThreeRight,       KBDK_J, KBDM_SHIFT},
+  {Event::JoystickThreeFire,        KBDK_F, KBDM_SHIFT},
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
