@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -19,11 +19,10 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ConvolutionBuffer::ConvolutionBuffer(uInt32 size)
-  : myFirstIndex(0),
-    mySize(size)
+  : mySize(size)
 {
   myData = make_unique<float[]>(mySize);
-  memset(myData.get(), 0, mySize * sizeof(float));
+  std::fill_n(myData.get(), mySize, 0.F);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -36,7 +35,7 @@ void ConvolutionBuffer::shift(float nextValue)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 float ConvolutionBuffer::convoluteWith(float* kernel) const
 {
-  float result = 0.;
+  float result = 0.F;
 
   for (uInt32 i = 0; i < mySize; ++i) {
     result += kernel[i] * myData[(myFirstIndex + i) % mySize];
