@@ -167,7 +167,7 @@ string Debugger::autoExec(StringList* history)
   ostringstream buf;
 
   // autoexec.script is always run
-  FilesystemNode autoexec(myOSystem.baseDir() + "autoexec.script");
+  FilesystemNode autoexec(myOSystem.baseDir().getPath() + "autoexec.script");
   buf << "autoExec():" << endl
       << myParser->exec(autoexec, history) << endl;
 
@@ -304,7 +304,7 @@ int Debugger::step(bool save)
   myOSystem.console().tia().updateScanlineByStep().flushLineCache();
   lockSystem();
 
-  if(save)  
+  if(save)
     addState("step");
   return int(mySystem.cycles() - startCycle);
 }
@@ -877,23 +877,26 @@ std::array<Debugger::BuiltinFunction, 18> Debugger::ourBuiltinFunctions = { {
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // Names are defined here, but processed in YaccParser
-std::array<Debugger::PseudoRegister, 12> Debugger::ourPseudoRegisters = { {
+std::array<Debugger::PseudoRegister, 14> Debugger::ourPseudoRegisters = { {
 // Debugger::PseudoRegister Debugger::ourPseudoRegisters[NUM_PSEUDO_REGS] = {
-  { "_bank",      "Currently selected bank" },
-  { "_cclocks",   "Color clocks on current scanline" },
-  { "_cycleshi",  "Higher 32 bits of number of cycles since emulation started" },
-  { "_cycleslo",  "Lower 32 bits of number of cycles since emulation started" },
-  { "_fcount",    "Number of frames since emulation started" },
-  { "_fcycles",   "Number of cycles since frame started" },
-  { "_icycles",   "Number of cycles of last instruction" },
-  { "_scan",      "Current scanline count" },
-  { "_scanend",   "Scanline count at end of last frame" },
-  { "_scycles",   "Number of cycles in current scanline" },
-  { "_vblank",    "Whether vertical blank is enabled (1 or 0)" },
-  { "_vsync",     "Whether vertical sync is enabled (1 or 0)" }
+  { "_bank",          "Currently selected bank" },
+  { "_cclocks",       "Color clocks on current scanline" },
+  { "_cycleshi",      "Higher 32 bits of number of cycles since emulation started" },
+  { "_cycleslo",      "Lower 32 bits of number of cycles since emulation started" },
+  { "_fcount",        "Number of frames since emulation started" },
+  { "_fcycles",       "Number of cycles since frame started" },
+  { "_icycles",       "Number of cycles of last instruction" },
+  { "_scan",          "Current scanline count" },
+  { "_scanend",       "Scanline count at end of last frame" },
+  { "_scycles",       "Number of cycles in current scanline" },
+  { "_timwrapread",   "Timer read wrapped on this cycle" },
+  { "_timwrapwrite",  "Timer write wrapped on this cycle" },
+  { "_vblank",        "Whether vertical blank is enabled (1 or 0)" },
+  { "_vsync",         "Whether vertical sync is enabled (1 or 0)" }
   // CPU address access functions:
   /*{ "_lastread", "last CPU read address" },
   { "_lastwrite", "last CPU write address" },
   { "__lastbaseread", "last CPU read base address" },
   { "__lastbasewrite", "last CPU write base address" }*/
 } };
+//
