@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -28,14 +28,6 @@ TIADebug::TIADebug(Debugger& dbg, Console& console)
   : DebuggerSystem(dbg, console),
     myTIA(console.tia())
 {
-  nusizStrings[0] = "1 copy";
-  nusizStrings[1] = "2 copies - close (8)";
-  nusizStrings[2] = "2 copies - med (24)";
-  nusizStrings[3] = "3 copies - close (8)";
-  nusizStrings[4] = "2 copies - wide (56)";
-  nusizStrings[5] = "2x (16) sized player";
-  nusizStrings[6] = "3 copies - med (24)";
-  nusizStrings[7] = "4x (32) sized player";
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -49,8 +41,8 @@ const DebuggerState& TIADebug::getState()
   myState.coluRegs.push_back(coluBK());
 
   // Debug Colors
-  int timing = myTIA.consoleTiming() == ConsoleTiming::ntsc ? 0
-    : myTIA.consoleTiming() == ConsoleTiming::pal ? 1 : 2;
+  int timing = myConsole.timing() == ConsoleTiming::ntsc ? 0
+    : myConsole.timing() == ConsoleTiming::pal ? 1 : 2;
 
   myState.fixedCols.clear();
   myState.fixedCols.push_back(myTIA.myFixedColorPalette[timing][TIA::P0]);
@@ -134,6 +126,11 @@ const DebuggerState& TIADebug::getState()
   myState.size.push_back(nusizM1());
   myState.size.push_back(sizeBL());
 
+  // VSync/VBlank registers
+  myState.vsb.clear();
+  myState.vsb.push_back(vsync());
+  myState.vsb.push_back(vblank());
+
   // Audio registers
   myState.aud.clear();
   myState.aud.push_back(audF0());
@@ -147,8 +144,8 @@ const DebuggerState& TIADebug::getState()
   myState.info.clear();
   myState.info.push_back(frameCount());
   myState.info.push_back(frameCycles());
-  myState.info.push_back(vsyncAsInt());
-  myState.info.push_back(vblankAsInt());
+  myState.info.push_back(cyclesLo());
+  myState.info.push_back(cyclesHi());
   myState.info.push_back(scanlines());
   myState.info.push_back(scanlinesLastFrame());
   myState.info.push_back(clocksThisLine());
@@ -238,6 +235,11 @@ void TIADebug::saveOldState()
   myOldState.size.push_back(nusizM1());
   myOldState.size.push_back(sizeBL());
 
+  // VSync/VBlank registers
+  myOldState.vsb.clear();
+  myOldState.vsb.push_back(vsync());
+  myOldState.vsb.push_back(vblank());
+
   // Audio registers
   myOldState.aud.clear();
   myOldState.aud.push_back(audF0());
@@ -251,8 +253,8 @@ void TIADebug::saveOldState()
   myOldState.info.clear();
   myOldState.info.push_back(frameCount());
   myOldState.info.push_back(frameCycles());
-  myOldState.info.push_back(vsyncAsInt());
-  myOldState.info.push_back(vblankAsInt());
+  myOldState.info.push_back(cyclesLo());
+  myOldState.info.push_back(cyclesHi());
   myOldState.info.push_back(scanlines());
   myOldState.info.push_back(scanlinesLastFrame());
   myOldState.info.push_back(clocksThisLine());
@@ -349,7 +351,7 @@ bool TIADebug::resMP1(int newVal)
   if(newVal > -1)
     mySystem.poke(RESMP1, bool(newVal) << 1);
 
-  return myTIA.registerValue(RESMP1) & 0x02;;
+  return myTIA.registerValue(RESMP1) & 0x02;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -441,7 +443,7 @@ bool TIADebug::collision(CollisionBit id, bool toggle) const
     case CollisionBit::M1P1:
       if(toggle)
         myTIA.toggleCollP1M1();
-      return myTIA.collCXM1P() & 0x40;
+      return myTIA.collCXM1P()  & 0x40;
 
     case CollisionBit::P0PF:
       if(toggle)
@@ -943,12 +945,30 @@ int TIADebug::cyclesThisLine() const
   return myTIA.clocksThisLine()/3;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool TIADebug::vsync(int newVal)
+{
+  if (newVal > -1)
+    mySystem.poke(VSYNC, newVal);
+
+  return myTIA.registerValue(VSYNC) & 0x02;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIADebug::vsync() const
 {
   return myTIA.registerValue(VSYNC) & 0x02;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool TIADebug::vblank(int newVal)
+{
+  if (newVal > -1)
+    mySystem.poke(VBLANK, newVal);
+
+  return myTIA.registerValue(VBLANK) & 0x02;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool TIADebug::vblank() const
 {
@@ -975,15 +995,16 @@ string TIADebug::colorSwatch(uInt8 c) const
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // FIXME - how does this work; is this even needed ??
+//         convert to stringstream, get rid of snprintf
 string TIADebug::audFreq(uInt8 div)
 {
   string ret;
-  char buf[10];
+  std::array<char, 10> buf;
 
   double hz = 31400.0;
   if(div) hz /= div;
-  std::snprintf(buf, 9, "%5.1f", hz);
-  ret += buf;
+  std::snprintf(buf.data(), 9, "%5.1f", hz);
+  ret += buf.data();
   ret += "Hz";
 
   return ret;
@@ -992,21 +1013,10 @@ string TIADebug::audFreq(uInt8 div)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string TIADebug::booleanWithLabel(string label, bool value)
 {
-  char buf[64];
-  string ret;
-
   if(value)
-  {
-    char *p = buf;
-    const char *q = label.c_str();
-    while((*p++ = toupper(*q++)))
-      ;
-    ret += "+";
-    ret += buf;
-    return ret;
-  }
+    return "+" + BSPF::toUpperCase(label);
   else
-    return "-" + label;
+    return "-" + BSPF::toLowerCase(label);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1014,8 +1024,8 @@ string TIADebug::debugColors() const
 {
   ostringstream buf;
 
-  int timing = myTIA.consoleTiming() == ConsoleTiming::ntsc ? 0
-    : myTIA.consoleTiming() == ConsoleTiming::pal ? 1 : 2;
+  int timing = myConsole.timing() == ConsoleTiming::ntsc ? 0
+    : myConsole.timing() == ConsoleTiming::pal ? 1 : 2;
 
   buf << " " << myTIA.myFixedColorNames[TIA::P0] << " " << colorSwatch(myTIA.myFixedColorPalette[timing][TIA::P0])
       << " Player 0\n"
@@ -1096,43 +1106,43 @@ string TIADebug::toString()
       << " BK=$" << Common::Base::HEX2 << state.coluRegs[3] << "/"
       << colorSwatch(state.coluRegs[3])
       << endl
-      << "P0: GR=%" << Common::Base::toString(state.gr[P0], Common::Base::F_2_8)
-      << " pos=#" << std::dec << state.pos[P0]
-      << " HM=$" << Common::Base::HEX2 << state.hm[P0] << " "
+      << "P0: GR=%" << Common::Base::toString(state.gr[TiaState::P0], Common::Base::Fmt::_2_8)
+      << " pos=#" << std::dec << state.pos[TiaState::P0]
+      << " HM=$" << Common::Base::HEX2 << state.hm[TiaState::P0] << " "
       << nusizP0String() << " "
       << booleanWithLabel("refl", refP0()) << " "
       << booleanWithLabel("delay", vdelP0())
       << endl
-      << "P1: GR=%" << Common::Base::toString(state.gr[P1], Common::Base::F_2_8)
-      << " pos=#" << std::dec << state.pos[P1]
-      << " HM=$" << Common::Base::HEX2 << state.hm[P1] << " "
+      << "P1: GR=%" << Common::Base::toString(state.gr[TiaState::P1], Common::Base::Fmt::_2_8)
+      << " pos=#" << std::dec << state.pos[TiaState::P1]
+      << " HM=$" << Common::Base::HEX2 << state.hm[TiaState::P1] << " "
       << nusizP1String() << " "
       << booleanWithLabel("refl", refP1()) << " "
       << booleanWithLabel("delay", vdelP1())
       << endl
       << "M0: " << (enaM0() ? " ENABLED" : "disabled")
-      << " pos=#" << std::dec << state.pos[M0]
-      << " HM=$" << Common::Base::HEX2 << state.hm[M0]
-      << " size=" << std::dec << state.size[M0] << " "
+      << " pos=#" << std::dec << state.pos[TiaState::M0]
+      << " HM=$" << Common::Base::HEX2 << state.hm[TiaState::M0]
+      << " size=" << std::dec << state.size[TiaState::M0] << " "
       << booleanWithLabel("reset", resMP0())
       << endl
       << "M1: " << (enaM1() ? " ENABLED" : "disabled")
-      << " pos=#" << std::dec << state.pos[M1]
-      << " HM=$" << Common::Base::HEX2 << state.hm[M1]
-      << " size=" << std::dec << state.size[M1] << " "
+      << " pos=#" << std::dec << state.pos[TiaState::M1]
+      << " HM=$" << Common::Base::HEX2 << state.hm[TiaState::M1]
+      << " size=" << std::dec << state.size[TiaState::M1] << " "
       << booleanWithLabel("reset", resMP0())
       << endl
       << "BL: " << (enaBL() ? " ENABLED" : "disabled")
-      << " pos=#" << std::dec << state.pos[BL]
-      << " HM=$" << Common::Base::HEX2 << state.hm[BL]
-      << " size=" << std::dec << state.size[BL] << " "
+      << " pos=#" << std::dec << state.pos[TiaState::BL]
+      << " HM=$" << Common::Base::HEX2 << state.hm[TiaState::BL]
+      << " size=" << std::dec << state.size[TiaState::BL] << " "
       << booleanWithLabel("delay", vdelBL())
       << endl
-      << "PF0: %" << Common::Base::toString(state.pf[0], Common::Base::F_2_8) << "/$"
+      << "PF0: %" << Common::Base::toString(state.pf[0], Common::Base::Fmt::_2_8) << "/$"
       << Common::Base::HEX2 << state.pf[0]
-      << " PF1: %" << Common::Base::toString(state.pf[1], Common::Base::F_2_8) << "/$"
+      << " PF1: %" << Common::Base::toString(state.pf[1], Common::Base::Fmt::_2_8) << "/$"
       << Common::Base::HEX2 << state.pf[1]
-      << " PF2: %" << Common::Base::toString(state.pf[2], Common::Base::F_2_8) << "/$"
+      << " PF2: %" << Common::Base::toString(state.pf[2], Common::Base::Fmt::_2_8) << "/$"
       << Common::Base::HEX2 << state.pf[2]
       << endl << "     "
       << booleanWithLabel("reflect",  refPF()) << " "
@@ -1171,3 +1181,15 @@ string TIADebug::toString()
   // note: last line should not contain \n, caller will add.
   return buf.str();
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+const std::array<string, 8> TIADebug::nusizStrings = {
+  "1 copy",
+  "2 copies - close (8)",
+  "2 copies - med (24)",
+  "3 copies - close (8)",
+  "2 copies - wide (56)",
+  "2x (16) sized player",
+  "3 copies - med (24)",
+  "4x (32) sized player"
+};
