@@ -39,6 +39,7 @@
 #include "AmigaMouseWidget.hxx"
 #include "AtariMouseWidget.hxx"
 #include "TrakBallWidget.hxx"
+#include "QuadTariWidget.hxx"
 
 #include "RiotWidget.hxx"
 
@@ -98,7 +99,7 @@ RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& lfont,
 
   // SWCHB bits in 'peek' mode
   xpos = 10;  ypos += lineHeight + 5;
-  CREATE_IO_REGS("SWCHB(R)", mySWCHBReadBits, 0, false)
+  CREATE_IO_REGS("SWCHB(R)", mySWCHBReadBits, kSWCHBRBitsID, true)
 
   // Timer registers (R/W)
   static constexpr std::array<const char*, 4> writeNames = {
@@ -140,9 +141,11 @@ RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& lfont,
   xpos = col;  ypos = 10;
   myLeftControl = addControlWidget(boss, lfont, xpos, ypos,
       instance().console().leftController());
+  addToFocusList(myLeftControl->getFocusList());
   xpos += myLeftControl->getWidth() + 15;
   myRightControl = addControlWidget(boss, lfont, xpos, ypos,
       instance().console().rightController());
+  addToFocusList(myRightControl->getFocusList());
 
   // TIA INPTx registers (R), left port
   static constexpr std::array<const char*, 3> contLeftReadNames = {
@@ -417,6 +420,17 @@ void RiotWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
           rport.setPin(Controller::DigitalPin::Four,  value & 0b00001000);
           break;
         }
+        case kSWCHBRBitsID:
+        {
+          value = Debugger::get_bits(mySWCHBReadBits->getState());
+
+          riot.reset( value & 0b00000001);
+          riot.select(value & 0b00000010);
+          riot.tvType(value & 0b00001000);
+          riot.diffP0(value & 0b01000000);
+          riot.diffP1(value & 0b10000000);
+          break;
+        }
         default:
           break;
       }
@@ -480,12 +494,15 @@ ControllerWidget* RiotWidget::addControlWidget(GuiObject* boss, const GUI::Font&
       return new KeyboardWidget(boss, font, x, y, controller);
 //    case Controller::Type::KidVid:      // TODO - implement this
 //    case Controller::Type::MindLink:    // TODO - implement this
+//    case Controller::Type::Lightgun:    // TODO - implement this
     case Controller::Type::Paddles:
       return new PaddleWidget(boss, font, x, y, controller);
     case Controller::Type::SaveKey:
       return new SaveKeyWidget(boss, font, x, y, controller);
     case Controller::Type::TrakBall:
       return new TrakBallWidget(boss, font, x, y, controller);
+    case Controller::Type::QuadTari:
+      return new QuadTariWidget(boss, font, x, y, controller);
     default:
       return new NullControlWidget(boss, font, x, y, controller);
   }
