@@ -20,8 +20,9 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeBFSC::CartridgeBFSC(const ByteBuffer& image, size_t size,
-                             const string& md5, const Settings& settings)
-  : CartridgeBF(image, size, md5, settings)
+                             const string& md5, const Settings& settings,
+                             size_t bsSize)
+  : CartridgeBF(image, size, md5, settings, bsSize)
 {
   myRamSize = RAM_SIZE;
 }
