@@ -29,14 +29,17 @@
 #include "TIA.hxx"
 #include "exception/FatalEmulationError.hxx"
 
-#define DSRAM         0x0800
-
 #define COMMSTREAM        0x20
 #define JUMPSTREAM_BASE   0x21
 
 #define FAST_FETCH_ON ((myMode & 0x0F) == 0)
 #define DIGITAL_AUDIO_ON ((myMode & 0xF0) == 0)
 
+#define getUInt32(_array, _address) ((_array)[(_address) + 0]        +  \
+                                    ((_array)[(_address) + 1] << 8)  +  \
+                                    ((_array)[(_address) + 2] << 16) +  \
+                                    ((_array)[(_address) + 3] << 24))
+
 namespace {
   Thumbulator::ConfigureFor thumulatorConfiguration(CartridgeCDF::CDFSubtype subtype)
   {
@@ -50,6 +53,9 @@ namespace {
       case CartridgeCDF::CDFSubtype::CDFJ:
         return Thumbulator::ConfigureFor::CDFJ;
 
+      case CartridgeCDF::CDFSubtype::CDFJplus:
+        return Thumbulator::ConfigureFor::CDFJplus;
+
       default:
         throw runtime_error("unreachable");
     }
@@ -59,33 +65,51 @@ namespace {
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeCDF::CartridgeCDF(const ByteBuffer& image, size_t size,
                            const string& md5, const Settings& settings)
-  : Cartridge(settings, md5)
+  : Cartridge(settings, md5),
+    myImage(make_unique<uInt8[]>(512_KB))
 {
   // Copy the ROM image into my buffer
-  std::copy_n(image.get(), std::min(myImage.size(), size), myImage.begin());
+  std::fill_n(myImage.get(), 512_KB, 0);
+  std::copy_n(image.get(), std::min(512_KB, size), myImage.get());
 
-  // even though the ROM is 32K, only 28K is accessible to the 6507
-  createRomAccessArrays(28_KB);
+  // Detect cart version
+  setupVersion();
 
-  // Pointer to the program ROM (28K @ 0 byte offset)
-  // which starts after the 2K CDF Driver and 2K C Code
-  myProgramImage = myImage.data() + 4_KB;
+  // The lowest 2K is not accessible to the debugger
+  createRomAccessArrays(isCDFJplus() ? 510_KB : 28_KB);
+
+  // Pointer to the program ROM
+  // which starts after the 2K driver (and 2K C Code for CDF)
+  myProgramImage = myImage.get() + (isCDFJplus() ? 2_KB : 4_KB);
 
   // Pointer to CDF driver in RAM
   myDriverImage = myRAM.data();
 
-  // Pointer to the display RAM
-  myDisplayImage = myRAM.data() + DSRAM;
+  // Pointer to the display RAM (starts after 2K driver)
+  myDisplayImage = myRAM.data() + 2_KB;
 
-  setupVersion();
+  // C addresses
+  uInt32 cBase, cStart, cStack;
+  if (isCDFJplus()) {
+    cBase = getUInt32(myImage.get(), 0x17F8) & 0xFFFFFFFE;    // C Base Address
+    cStart = cBase;                                           // C Start Address
+    cStack = getUInt32(myImage.get(), 0x17F4);                // C Stack
+  } else {
+    cBase = 0x800;          // C Base Address
+    cStart = 0x808;         // C Start Address (skip ARM header)
+    cStack = 0x40001FDC;    // C Stack
+  }
 
   // Create Thumbulator ARM emulator
   bool devSettings = settings.getBool("dev.settings");
   myThumbEmulator = make_unique<Thumbulator>(
-    reinterpret_cast<uInt16*>(myImage.data()),
+    reinterpret_cast<uInt16*>(myImage.get()),
     reinterpret_cast<uInt16*>(myRAM.data()),
-    static_cast<uInt32>(myImage.size()),
-    devSettings ? settings.getBool("dev.thumb.trapfatal") : false, thumulatorConfiguration(myCDFSubtype), this);
+    static_cast<uInt32>(512_KB),
+    cBase, cStart, cStack,
+    devSettings ? settings.getBool("dev.thumb.trapfatal") : false, 
+    thumulatorConfiguration(myCDFSubtype), 
+    this);
 
   setInitialState();
 }
@@ -95,8 +119,8 @@ void CartridgeCDF::reset()
 {
   initializeRAM(myRAM.data()+2_KB, myRAM.size()-2_KB);
 
-  // CDF always starts in bank 6
-  initializeStartBank(6);
+  // CDF always starts in bank 6, CDFJ+ in bank 0
+  initializeStartBank(isCDFJplus() ? 0 : 6);
 
   myAudioCycles = myARMCycles = 0;
   myFractionalClocks = 0.0;
@@ -111,7 +135,7 @@ void CartridgeCDF::reset()
 void CartridgeCDF::setInitialState()
 {
   // Copy initial CDF driver to Harmony RAM
-  std::copy_n(myImage.begin(), 2_KB, myDriverImage);
+  std::copy_n(myImage.get(), 2_KB, myDriverImage);
 
   myMusicWaveformSize.fill(27);
 
@@ -121,7 +145,6 @@ void CartridgeCDF::setInitialState()
 
   myBankOffset = myLDAimmediateOperandAddress = myJMPoperandAddress = 0;
   myFastJumpActive = myFastJumpStream = 0;
-
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -193,7 +216,6 @@ inline void CartridgeCDF::callFunction(uInt8 value)
 uInt8 CartridgeCDF::peek(uInt16 address)
 {
   address &= 0x0FFF;
-
   uInt8 peekvalue = myProgramImage[myBankOffset + address];
 
   // In debugger/bank-locked mode, we ignore all hotspots and in general
@@ -212,8 +234,14 @@ uInt8 CartridgeCDF::peek(uInt16 address)
     ++myJMPoperandAddress;
 
     pointer = getDatastreamPointer(myFastJumpStream);
-    value = myDisplayImage[ pointer >> 20 ];
-    pointer += 0x100000;  // always increment by 1
+    if (isCDFJplus()) {
+      value = myDisplayImage[ pointer >> 16 ];
+      pointer += 0x00010000;  // always increment by 1
+    } else {
+      value = myDisplayImage[ pointer >> 20 ];
+      pointer += 0x00100000;  // always increment by 1
+    }
+
     setDatastreamPointer(myFastJumpStream, pointer);
 
     return value;
@@ -252,9 +280,9 @@ uInt8 CartridgeCDF::peek(uInt16 address)
         uInt32 sampleaddress = getSample() + (myMusicCounters[0] >> 21);
 
         // get sample value from ROM or RAM
-        if (sampleaddress < 0x8000)
+        if (sampleaddress < 0x00080000)
           peekvalue = myImage[sampleaddress];
-        else if (sampleaddress >= 0x40000000 && sampleaddress < 0x40002000) // check for RAM
+        else if (sampleaddress >= 0x40000000 && sampleaddress < 0x40008000) // check for RAM
           peekvalue = myRAM[sampleaddress - 0x40000000];
         else
           peekvalue = 0;
@@ -282,39 +310,36 @@ uInt8 CartridgeCDF::peek(uInt16 address)
   // Switch banks if necessary
   switch(address)
   {
-    case 0xFF5:
-      // Set the current bank to the first 4k bank
-      bank(0);
+    case 0x0FF4:
+      bank(isCDFJplus() ? 0 : 6);
+      break;
+
+    case 0x0FF5:
+      bank(isCDFJplus() ? 1 : 0);
       break;
 
     case 0x0FF6:
-      // Set the current bank to the second 4k bank
-      bank(1);
+      bank(isCDFJplus() ? 2 : 1);
       break;
 
     case 0x0FF7:
-      // Set the current bank to the third 4k bank
-      bank(2);
+      bank(isCDFJplus() ? 3 : 2);
       break;
 
     case 0x0FF8:
-      // Set the current bank to the fourth 4k bank
-      bank(3);
+      bank(isCDFJplus() ? 4 : 3);
       break;
 
     case 0x0FF9:
-      // Set the current bank to the fifth 4k bank
-      bank(4);
+      bank(isCDFJplus() ? 5 : 4);
       break;
 
     case 0x0FFA:
-      // Set the current bank to the sixth 4k bank
-      bank(5);
+      bank(isCDFJplus() ? 6 : 5);
       break;
 
     case 0x0FFB:
-      // Set the current bank to the last 4k bank
-      bank(6);
+      bank(isCDFJplus() ? 0 : 6);
       break;
 
     default:
@@ -333,65 +358,71 @@ bool CartridgeCDF::poke(uInt16 address, uInt8 value)
   uInt32 pointer;
 
   address &= 0x0FFF;
-
   switch(address)
   {
-    case 0xFF0:   // DSWRITE
+    case 0x0FF0:   // DSWRITE
       pointer = getDatastreamPointer(COMMSTREAM);
-      myDisplayImage[ pointer >> 20 ] = value;
-      pointer += 0x100000;  // always increment by 1 when writing
+      if (isCDFJplus()) {
+        myDisplayImage[ pointer >> 16 ] = value;
+        pointer += 0x00010000;  // always increment by 1 when writing  
+      } else {
+        myDisplayImage[ pointer >> 20 ] = value;
+        pointer += 0x00100000;  // always increment by 1 when writing
+      }
       setDatastreamPointer(COMMSTREAM, pointer);
       break;
 
-    case 0xFF1:   // DSPTR
+    case 0x0FF1:   // DSPTR
       pointer = getDatastreamPointer(COMMSTREAM);
-      pointer <<=8;
-      pointer &= 0xf0000000;
-      pointer |= (value << 20);
+      pointer <<= 8;
+      if (isCDFJplus()) {
+        pointer &= 0xff000000;
+        pointer |= (value << 16);
+      } else {
+        pointer &= 0xf0000000;
+        pointer |= (value << 20);
+      }
       setDatastreamPointer(COMMSTREAM, pointer);
       break;
 
-    case 0xFF2:   // SETMODE
+    case 0x0FF2:   // SETMODE
       myMode = value;
       break;
 
-    case 0xFF3:   // CALLFN
+    case 0x0FF3:   // CALLFN
       callFunction(value);
       break;
 
-    case 0xFF5:
-      // Set the current bank to the first 4k bank
-      bank(0);
+   case 0x00FF4:
+      bank(isCDFJplus() ? 0 : 6);
+      break;
+
+    case 0x0FF5:
+      bank(isCDFJplus() ? 1 : 0);
       break;
 
     case 0x0FF6:
-      // Set the current bank to the second 4k bank
-      bank(1);
+      bank(isCDFJplus() ? 2 : 1);
       break;
 
     case 0x0FF7:
-      // Set the current bank to the third 4k bank
-      bank(2);
+      bank(isCDFJplus() ? 3 : 2);
       break;
 
     case 0x0FF8:
-      // Set the current bank to the fourth 4k bank
-      bank(3);
+      bank(isCDFJplus() ? 4 : 3);
       break;
 
     case 0x0FF9:
-      // Set the current bank to the fifth 4k bank
-      bank(4);
+      bank(isCDFJplus() ? 5 : 4);
       break;
 
     case 0x0FFA:
-      // Set the current bank to the sixth 4k bank
-      bank(5);
+      bank(isCDFJplus() ? 6 : 5);
       break;
 
     case 0x0FFB:
-      // Set the current bank to the last 4k bank
-      bank(6);
+      bank(isCDFJplus() ? 0 : 6);
       break;
 
     default:
@@ -417,7 +448,7 @@ bool CartridgeCDF::bank(uInt16 bank, uInt16)
   {
     access.romAccessBase = &myRomAccessBase[myBankOffset + (addr & 0x0FFF)];
     access.romPeekCounter = &myRomAccessCounter[myBankOffset + (addr & 0x0FFF)];
-    access.romPokeCounter = &myRomAccessCounter[myBankOffset + (addr & 0x0FFF) + 28_KB];
+    access.romPokeCounter = &myRomAccessCounter[myBankOffset + (addr & 0x0FFF) + 28_KB];  // TODO: Change for CDFJ+???
     mySystem->setPageAccess(addr, access);
   }
   return myBankChanged = true;
@@ -451,10 +482,10 @@ bool CartridgeCDF::patch(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* CartridgeCDF::getImage(size_t& size) const
+const ByteBuffer& CartridgeCDF::getImage(size_t& size) const
 {
-  size = myImage.size();
-  return myImage.data();
+  size = 512_KB;
+  return myImage;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -615,11 +646,13 @@ uInt32 CartridgeCDF::getWaveform(uInt8 index) const
                  (myRAM[address + 2] << 16) +
                  (myRAM[address + 3] << 24);   // high byte
 
-  result -= (0x40000000 + DSRAM);
-
-  if (result >= 4096)
-    result &= 4095;
+  result -= (0x40000000 + uInt32(2_KB));
 
+  if (!isCDFJplus()) {
+    if (result >= 4096) {
+      result &= 4095;
+    }
+  }
   return result;
 }
 
@@ -657,8 +690,16 @@ uInt8 CartridgeCDF::readFromDatastream(uInt8 index)
 
   uInt32 pointer = getDatastreamPointer(index);
   uInt16 increment = getDatastreamIncrement(index);
-  uInt8 value = myDisplayImage[ pointer >> 20 ];
-  pointer += (increment << 12);
+
+  uInt8 value;
+  if (isCDFJplus()) {
+    value = myDisplayImage[ pointer >> 16 ];
+    pointer += (increment << 8);
+  } else {
+    value = myDisplayImage[ pointer >> 20 ];
+    pointer += (increment << 12);   
+  }
+
   setDatastreamPointer(index, pointer);
   return value;
 }
@@ -666,8 +707,21 @@ uInt8 CartridgeCDF::readFromDatastream(uInt8 index)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeCDF::setupVersion()
 {
-  uInt8 subversion = 0;
+  // CDFJ+ detection
+  if (getUInt32(myImage.get(), 0x174) == 0x53554c50 &&    // Plus
+      getUInt32(myImage.get(), 0x178) == 0x4a464443 &&    // CDFJ
+      getUInt32(myImage.get(), 0x17C) == 0x00000001) {    // V1
 
+    myCDFSubtype = CDFSubtype::CDFJplus;
+    myAmplitudeStream = 0x23;
+    myFastjumpStreamIndexMask = 0xfe;
+    myDatastreamBase = 0x0098;
+    myDatastreamIncrementBase = 0x0124;
+    myWaveformBase = 0x01b0;
+    return;
+  }
+
+  uInt8 subversion = 0;
   for(uInt32 i = 0; i < 2048; i += 4)
   {
     // CDF signature occurs 3 times in a row, i+3 (+7 or +11) is version
@@ -681,6 +735,7 @@ void CartridgeCDF::setupVersion()
   }
 
   switch (subversion) {
+
     case 0x4a:
       myCDFSubtype = CDFSubtype::CDFJ;
 
@@ -725,11 +780,29 @@ string CartridgeCDF::name() const
       return "CartridgeCDF1";
     case CDFSubtype::CDFJ:
       return "CartridgeCDFJ";
+    case CDFSubtype::CDFJplus:
+      return "CartridgeCDFJ+";
     default:
       return "Cart unknown";
   }
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeCDF::isCDFJplus() const
+{
+  return (myCDFSubtype == CDFSubtype::CDFJplus);
+}
+
+uInt32 CartridgeCDF::ramSize() const
+{
+  return uInt32(isCDFJplus() ? 32_KB : 8_KB);
+}
+
+uInt32 CartridgeCDF::romSize() const
+{
+  return uInt32(isCDFJplus() ? 512_KB : 32_KB);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #ifdef DEBUGGER_SUPPORT
   CartDebugWidget* CartridgeCDF::debugWidget(GuiObject* boss, const GUI::Font& lfont,
