@@ -35,6 +35,11 @@
 #include "Genesis.hxx"
 #include "MindLink.hxx"
 #include "CompuMate.hxx"
+#include "AmigaMouse.hxx"
+#include "AtariMouse.hxx"
+#include "TrakBall.hxx"
+#include "Lightgun.hxx"
+#include "QuadTari.hxx"
 #include "M6502.hxx"
 #include "M6532.hxx"
 #include "TIA.hxx"
@@ -46,10 +51,6 @@
 #include "Sound.hxx"
 #include "Switches.hxx"
 #include "System.hxx"
-#include "AmigaMouse.hxx"
-#include "AtariMouse.hxx"
-#include "TrakBall.hxx"
-#include "Lightgun.hxx"
 #include "FrameBuffer.hxx"
 #include "TIASurface.hxx"
 #include "OSystem.hxx"
@@ -73,6 +74,45 @@
 
 #include "Console.hxx"
 
+namespace {
+  // Emulation speed is a positive float that multiplies the framerate. However, the UI controls
+  // adjust speed in terms of a speedup factor (1/10, 1/9 .. 1/2, 1, 2, 3, .., 10). The following
+  // mapping and formatting functions implement this conversion. The speedup factor is represented
+  // by an integer value between -900 and 900 (0 means no speedup).
+
+  constexpr int MAX_SPEED = 900;
+  constexpr int MIN_SPEED = -900;
+  constexpr int SPEED_STEP = 10;
+
+  int mapSpeed(float speed)
+  {
+    speed = std::abs(speed);
+
+    return BSPF::clamp(
+      static_cast<int>(round(100 * (speed >= 1 ? speed - 1 : -1 / speed + 1))),
+      MIN_SPEED, MAX_SPEED
+    );
+  }
+
+  float unmapSpeed(int speed)
+  {
+    float f_speed = static_cast<float>(speed) / 100;
+
+    return speed < 0 ? -1 / (f_speed - 1) : 1 + f_speed;
+  }
+
+  string formatSpeed(int speed) {
+    stringstream ss;
+
+    ss
+      << std::setw(3) << std::fixed << std::setprecision(0)
+      << (unmapSpeed(speed) * 100);
+
+    return ss.str();
+  }
+}
+
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Console::Console(OSystem& osystem, unique_ptr<Cartridge>& cart,
                  const Properties& props, AudioSettings& audioSettings)
@@ -192,7 +232,7 @@ Console::Console(OSystem& osystem, unique_ptr<Cartridge>& cart,
   myConsoleInfo.BankSwitch = myCart->about();
 
   // Some carts have an associated nvram file
-  myCart->setNVRamFile(myOSystem.nvramDir(), myConsoleInfo.CartName);
+  myCart->setNVRamFile(myOSystem.nvramDir().getPath(), myConsoleInfo.CartName);
 
   // Let the other devices know about the new console
   mySystem->consoleChanged(myConsoleTiming);
@@ -434,7 +474,7 @@ void Console::setFormat(uInt32 format, bool force)
 
   setTIAProperties();
   initializeVideo();  // takes care of refreshing the screen
-  initializeAudio(); // ensure that audio synthesis is set up to match emulation speed
+  initializeAudio(); // ensure that audio synthesis is set up to match emulation rate
   myOSystem.resetFps(); // Reset FPS measurement
 
   myOSystem.frameBuffer().showMessage(message);
@@ -444,7 +484,7 @@ void Console::setFormat(uInt32 format, bool force)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Console::toggleColorLoss()
+void Console::toggleColorLoss(bool toggle)
 {
   bool colorloss = !myTIA->colorLossEnabled();
   if(myTIA->enableColorLoss(colorloss))
@@ -492,7 +532,7 @@ void Console::toggleTurbo()
 
   myOSystem.settings().setValue("turbo", !enabled);
 
-  // update speed
+  // update rate
   initializeAudio();
 
   // update VSync
@@ -503,6 +543,31 @@ void Console::toggleTurbo()
   myOSystem.frameBuffer().showMessage(ss.str());
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::changeSpeed(int direction)
+{
+  int speed = mapSpeed(myOSystem.settings().getFloat("speed"));
+  bool turbo = myOSystem.settings().getBool("turbo");
+
+  speed = BSPF::clamp(speed + direction * SPEED_STEP, MIN_SPEED, MAX_SPEED);
+  myOSystem.settings().setValue("speed", unmapSpeed(speed));
+
+  // update rate
+  initializeAudio();
+
+  if(turbo)
+  {
+    myOSystem.settings().setValue("turbo", false);
+    // update VSync
+    initializeVideo();
+  }
+
+  ostringstream val;
+
+  val << formatSpeed(speed) << "%";
+  myOSystem.frameBuffer().showMessage("Emulation speed", val.str(), speed, MIN_SPEED, MAX_SPEED);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Console::togglePhosphor()
 {
@@ -525,8 +590,11 @@ void Console::changePhosphor(int direction)
 {
   int blend = BSPF::stringToInt(myProperties.get(PropType::Display_PPBlend));
 
-  blend = BSPF::clamp(blend + direction * 2, 0, 100);
-  myOSystem.frameBuffer().tiaSurface().enablePhosphor(true, blend);
+  if(direction)
+  {
+    blend = BSPF::clamp(blend + direction * 2, 0, 100);
+    myOSystem.frameBuffer().tiaSurface().enablePhosphor(true, blend);
+  }
 
   ostringstream val;
   val << blend;
@@ -554,11 +622,20 @@ FBInitStatus Console::initializeVideo(bool full)
 
   if(full)
   {
+    uInt32 width, height;
+    if (!myOSystem.settings().getBool("tia.correct_aspect")) {
+      width =  2 * myTIA->width();
+      height = myTIA->height();
+    } else {
+      width = TIAConstants::viewableWidth;
+      height = TIAConstants::viewableHeight;
+    }
+
     bool devSettings = myOSystem.settings().getBool("dev.settings");
     const string& title = string("Stella ") + STELLA_VERSION +
                    ": \"" + myProperties.get(PropType::Cart_Name) + "\"";
     fbstatus = myOSystem.frameBuffer().createDisplay(title, FrameBuffer::BufferType::Emulator,
-        TIAConstants::viewableWidth, TIAConstants::viewableHeight, false);
+        width, height, false);
     if(fbstatus != FBInitStatus::Success)
       return fbstatus;
 
@@ -660,6 +737,22 @@ void Console::changeVSizeAdjust(int direction)
   myOSystem.frameBuffer().showMessage("V-Size", val.str(), newAdjustVSize, -5, 5);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::toggleCorrectAspectRatio(bool toggle)
+{
+  bool enabled = myOSystem.settings().getBool("tia.correct_aspect");
+
+  if(toggle)
+  {
+    enabled = !enabled;
+    myOSystem.settings().setValue("tia.correct_aspect", enabled);
+    initializeVideo();
+  }
+  const string message = string("Correct aspect ratio ") + (enabled ? "enabled" : "disabled");
+
+  myOSystem.frameBuffer().showMessage(message);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Console::setTIAProperties()
 {
@@ -726,7 +819,7 @@ void Console::setControllers(const string& romMd5)
     Controller::Type leftType = Controller::getType(myProperties.get(PropType::Controller_Left));
     Controller::Type rightType = Controller::getType(myProperties.get(PropType::Controller_Right));
     size_t size = 0;
-    const uInt8* image = myCart->getImage(size);
+    const ByteBuffer& image = myCart->getImage(size);
     const bool swappedPorts = myProperties.get(PropType::Console_SwapPorts) == "YES";
 
     // Try to detect controllers
@@ -821,7 +914,8 @@ unique_ptr<Controller> Console::getControllerPort(const Controller::Type type,
 
     case Controller::Type::AtariVox:
     {
-      const string& nvramfile = myOSystem.nvramDir() + "atarivox_eeprom.dat";
+      FilesystemNode nvramfile = myOSystem.nvramDir();
+      nvramfile /= "atarivox_eeprom.dat";
       Controller::onMessageCallback callback = [&os = myOSystem](const string& msg) {
         bool devSettings = os.settings().getBool("dev.settings");
         if(os.settings().getBool(devSettings ? "dev.eepromaccess" : "plr.eepromaccess"))
@@ -833,7 +927,8 @@ unique_ptr<Controller> Console::getControllerPort(const Controller::Type type,
     }
     case Controller::Type::SaveKey:
     {
-      const string& nvramfile = myOSystem.nvramDir() + "savekey_eeprom.dat";
+      FilesystemNode nvramfile = myOSystem.nvramDir();
+      nvramfile /= "savekey_eeprom.dat";
       Controller::onMessageCallback callback = [&os = myOSystem](const string& msg) {
         bool devSettings = os.settings().getBool("dev.settings");
         if(os.settings().getBool(devSettings ? "dev.eepromaccess" : "plr.eepromaccess"))
@@ -858,6 +953,10 @@ unique_ptr<Controller> Console::getControllerPort(const Controller::Type type,
       controller = make_unique<Lightgun>(port, myEvent, *mySystem, romMd5, myOSystem.frameBuffer());
       break;
 
+    case Controller::Type::QuadTari:
+      controller = make_unique<QuadTari>(port, myOSystem, *mySystem, myProperties);
+      break;
+
     default:
       // What else can we do?
       // always create because it may have been changed by user dialog
@@ -867,6 +966,29 @@ unique_ptr<Controller> Console::getControllerPort(const Controller::Type type,
   return controller;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::changeAutoFireRate(int direction)
+{
+  const Int32 scanlines = std::max<Int32>(tia().scanlinesLastFrame(), 240);
+  const bool isNTSC = scanlines <= 287;
+
+  int rate = myOSystem.settings().getInt("autofirerate");
+
+  rate = BSPF::clamp(rate + direction, 0, isNTSC ? 30 : 25);
+
+  myOSystem.settings().setValue("autofirerate", rate);
+  Controller::setAutoFireRate(rate);
+
+  ostringstream val;
+
+  if(rate)
+    val << rate << " Hz";
+  else
+    val << "Off";
+
+  myOSystem.frameBuffer().showMessage("Autofire rate", val.str(), rate, 0, isNTSC ? 30 : 25);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 float Console::getFramerate() const
 {
@@ -876,51 +998,56 @@ float Console::getFramerate() const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Console::toggleTIABit(TIABit bit, const string& bitname, bool show) const
+void Console::toggleTIABit(TIABit bit, const string& bitname, bool show, bool toggle) const
 {
-  bool result = myTIA->toggleBit(bit);
-  string message = bitname + (result ? " enabled" : " disabled");
+  bool result = myTIA->toggleBit(bit, toggle ? 2 : 3);
+  const string message = bitname + (result ? " enabled" : " disabled");
+
   myOSystem.frameBuffer().showMessage(message);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Console::toggleBits() const
+void Console::toggleBits(bool toggle) const
 {
-  bool enabled = myTIA->toggleBits();
-  string message = string("TIA bits") + (enabled ? " enabled" : " disabled");
+  bool enabled = myTIA->toggleBits(toggle);
+  const string message = string("TIA bits ") + (enabled ? "enabled" : "disabled");
+
   myOSystem.frameBuffer().showMessage(message);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Console::toggleTIACollision(TIABit bit, const string& bitname, bool show) const
+void Console::toggleTIACollision(TIABit bit, const string& bitname, bool show, bool toggle) const
 {
-  bool result = myTIA->toggleCollision(bit);
-  string message = bitname + (result ? " collision enabled" : " collision disabled");
+  bool result = myTIA->toggleCollision(bit, toggle ? 2 : 3);
+  const string message = bitname + (result ? " collision enabled" : " collision disabled");
+
   myOSystem.frameBuffer().showMessage(message);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Console::toggleCollisions() const
+void Console::toggleCollisions(bool toggle) const
 {
-  bool enabled = myTIA->toggleCollisions();
-  string message = string("TIA collisions") + (enabled ? " enabled" : " disabled");
+  bool enabled = myTIA->toggleCollisions(toggle);
+  const string message = string("TIA collisions ") + (enabled ? "enabled" : "disabled");
+
   myOSystem.frameBuffer().showMessage(message);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Console::toggleFixedColors() const
+void Console::toggleFixedColors(bool toggle) const
 {
-  if(myTIA->toggleFixedColors())
-    myOSystem.frameBuffer().showMessage("Fixed debug colors enabled");
-  else
-    myOSystem.frameBuffer().showMessage("Fixed debug colors disabled");
+  bool enabled = toggle ? myTIA->toggleFixedColors() : myTIA->usingFixedColors();
+  const string message = string("Fixed debug colors ") + (enabled ? "enabled" : "disabled");
+
+  myOSystem.frameBuffer().showMessage(message);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Console::toggleJitter() const
+void Console::toggleJitter(bool toggle) const
 {
-  bool enabled = myTIA->toggleJitter();
-  string message = string("TV scanline jitter") + (enabled ? " enabled" : " disabled");
+  bool enabled = myTIA->toggleJitter(toggle ? 2 : 3);
+  const string message = string("TV scanline jitter ") + (enabled ? "enabled" : "disabled");
+
   myOSystem.frameBuffer().showMessage(message);
 }
 
