@@ -22,8 +22,10 @@
 #include "ControllerDetector.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Controller::Type ControllerDetector::detectType(const uInt8* image, size_t size,
-    const Controller::Type type, const Controller::Jack port, const Settings& settings)
+Controller::Type ControllerDetector::detectType(
+    const ByteBuffer& image, size_t size,
+    const Controller::Type type, const Controller::Jack port,
+    const Settings& settings)
 {
   if(type == Controller::Type::Unknown || settings.getBool("rominfo"))
   {
@@ -43,7 +45,7 @@ Controller::Type ControllerDetector::detectType(const uInt8* image, size_t size,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string ControllerDetector::detectName(const uInt8* image, size_t size,
+string ControllerDetector::detectName(const ByteBuffer& image, size_t size,
     const Controller::Type controller, const Controller::Jack port,
     const Settings& settings)
 {
@@ -51,7 +53,8 @@ string ControllerDetector::detectName(const uInt8* image, size_t size,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Controller::Type ControllerDetector::autodetectPort(const uInt8* image, size_t size,
+Controller::Type ControllerDetector::autodetectPort(
+    const ByteBuffer& image, size_t size,
     Controller::Jack port, const Settings& settings)
 {
   // default type joystick
@@ -59,6 +62,8 @@ Controller::Type ControllerDetector::autodetectPort(const uInt8* image, size_t s
 
   if(isProbablySaveKey(image, size, port))
     type = Controller::Type::SaveKey;
+  else if(isProbablyQuadTari(image, size, port))
+    type = Controller::Type::QuadTari;
   else if(usesJoystickButton(image, size, port))
   {
     if(isProbablyTrakBall(image, size))
@@ -88,7 +93,7 @@ Controller::Type ControllerDetector::autodetectPort(const uInt8* image, size_t s
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool ControllerDetector::searchForBytes(const uInt8* image, size_t imagesize,
+bool ControllerDetector::searchForBytes(const ByteBuffer& image, size_t imagesize,
                                         const uInt8* signature, uInt32 sigsize)
 {
   if (imagesize >= sigsize)
@@ -112,13 +117,13 @@ bool ControllerDetector::searchForBytes(const uInt8* image, size_t imagesize,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool ControllerDetector::usesJoystickButton(const uInt8* image, size_t size,
+bool ControllerDetector::usesJoystickButton(const ByteBuffer& image, size_t size,
                                             Controller::Jack port)
 {
   if(port == Controller::Jack::Left)
   {
     // check for INPT4 access
-    const int NUM_SIGS_0 = 23;
+    const int NUM_SIGS_0 = 24;
     const int SIG_SIZE_0 = 3;
     uInt8 signature_0[NUM_SIGS_0][SIG_SIZE_0] = {
       { 0x24, 0x0c, 0x10 }, // bit INPT4; bpl (joystick games only)
@@ -136,6 +141,7 @@ bool ControllerDetector::usesJoystickButton(const uInt8* image, size_t size,
       { 0xb4, 0x0c, 0x30 }, // ldy INPT4|$30,x; bmi (joystick games only)
       { 0xa5, 0x3c, 0x2a }, // ldy INPT4|$30; rol (joystick games only)
       { 0xa6, 0x3c, 0x8e }, // ldx INPT4|$30; stx (joystick games only)
+      { 0xa6, 0x0c, 0x8e }, // ldx INPT4; stx (joystick games only)
       { 0xa4, 0x3c, 0x8c }, // ldy INPT4; sty (joystick games only, Scramble)
       { 0xa5, 0x0c, 0x8d }, // lda INPT4; sta (joystick games only, Super Cobra Arcade)
       { 0xa4, 0x0c, 0x30 }, // ldy INPT4|; bmi (only Game of Concentration)
@@ -242,7 +248,7 @@ bool ControllerDetector::usesJoystickButton(const uInt8* image, size_t size,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool ControllerDetector::usesKeyboard(const uInt8* image, size_t size,
+bool ControllerDetector::usesKeyboard(const ByteBuffer& image, size_t size,
                                       Controller::Jack port)
 {
   if(port == Controller::Jack::Left)
@@ -383,13 +389,13 @@ bool ControllerDetector::usesKeyboard(const uInt8* image, size_t size,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool ControllerDetector::usesGenesisButton(const uInt8* image, size_t size,
+bool ControllerDetector::usesGenesisButton(const ByteBuffer& image, size_t size,
                                            Controller::Jack port)
 {
   if(port == Controller::Jack::Left)
   {
     // check for INPT1 access
-    const int NUM_SIGS_0 = 18;
+    const int NUM_SIGS_0 = 19;
     const int SIG_SIZE_0 = 3;
     uInt8 signature_0[NUM_SIGS_0][SIG_SIZE_0] = {
       { 0x24, 0x09, 0x10 }, // bit INPT1; bpl (Genesis only)
@@ -405,6 +411,7 @@ bool ControllerDetector::usesGenesisButton(const uInt8* image, size_t size,
       { 0xa4, 0x39, 0x30 }, // ldy INPT1|$30; bmi (keyboard ROMS too)
       { 0xa5, 0x39, 0x6a }, // lda INPT1|$30; ror (Genesis only)
       { 0xa6, 0x39, 0x8e }, // ldx INPT1|$30; stx (Genesis only)
+      { 0xa6, 0x09, 0x8e }, // ldx INPT1; stx (Genesis only)
       { 0xa4, 0x39, 0x8c }, // ldy INPT1|$30; sty (Genesis only, Scramble)
       { 0xa5, 0x09, 0x8d }, // lda INPT1; sta (Genesis only, Super Cobra Arcade)
       { 0xa5, 0x09, 0x29 }, // lda INPT1; and (Genesis only)
@@ -440,7 +447,7 @@ bool ControllerDetector::usesGenesisButton(const uInt8* image, size_t size,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool ControllerDetector::usesPaddle(const uInt8* image, size_t size,
+bool ControllerDetector::usesPaddle(const ByteBuffer& image, size_t size,
                                     Controller::Jack port, const Settings& settings)
 {
   if(port == Controller::Jack::Left)
@@ -549,7 +556,7 @@ bool ControllerDetector::usesPaddle(const uInt8* image, size_t size,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool ControllerDetector::isProbablyTrakBall(const uInt8* image, size_t size)
+bool ControllerDetector::isProbablyTrakBall(const ByteBuffer& image, size_t size)
 {
   // check for TrakBall tables
   const int NUM_SIGS = 3;
@@ -568,7 +575,7 @@ bool ControllerDetector::isProbablyTrakBall(const uInt8* image, size_t size)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool ControllerDetector::isProbablyAtariMouse(const uInt8* image, size_t size)
+bool ControllerDetector::isProbablyAtariMouse(const ByteBuffer& image, size_t size)
 {
   // check for Atari Mouse tables
   const int NUM_SIGS = 3;
@@ -587,7 +594,7 @@ bool ControllerDetector::isProbablyAtariMouse(const uInt8* image, size_t size)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool ControllerDetector::isProbablyAmigaMouse(const uInt8* image, size_t size)
+bool ControllerDetector::isProbablyAmigaMouse(const ByteBuffer& image, size_t size)
 {
   // check for Amiga Mouse tables
   const int NUM_SIGS = 4;
@@ -607,7 +614,7 @@ bool ControllerDetector::isProbablyAmigaMouse(const uInt8* image, size_t size)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool ControllerDetector::isProbablySaveKey(const uInt8* image, size_t size,
+bool ControllerDetector::isProbablySaveKey(const ByteBuffer& image, size_t size,
                                            Controller::Jack port)
 {
   // check for known SaveKey code, only supports right port
@@ -652,7 +659,7 @@ bool ControllerDetector::isProbablySaveKey(const uInt8* image, size_t size,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool ControllerDetector::isProbablyLightGun(const uInt8* image, size_t size,
+bool ControllerDetector::isProbablyLightGun(const ByteBuffer& image, size_t size,
                                             Controller::Jack port)
 {
   if (port == Controller::Jack::Left)
@@ -685,6 +692,29 @@ bool ControllerDetector::isProbablyLightGun(const uInt8* image, size_t size,
       if (searchForBytes(image, size, signature[i], SIG_SIZE))
         return true;
   }
+  return false;
+}
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool ControllerDetector::isProbablyQuadTari(const ByteBuffer& image, size_t size,
+                                            Controller::Jack port)
+{
+  uInt8 signatureBoth[] = { 0x1B, 0x1F, 0x0B, 0x0E, 0x1E, 0x0B, 0x1C, 0x13 }; // "QUADTARI"
+
+  if(searchForBytes(image, size, signatureBoth, 8))
+    return true;
+
+  if(port == Controller::Jack::Left)
+  {
+    uInt8 signature[] = { 'Q', 'U', 'A', 'D', 'L' };
+
+    return searchForBytes(image, size, signature, 5);
+  }
+  else if(port == Controller::Jack::Right)
+  {
+    uInt8 signature[] = { 'Q', 'U', 'A', 'D', 'R' };
+
+    return searchForBytes(image, size, signature, 5);
+  }
   return false;
 }
