@@ -29,6 +29,8 @@
 #include "FBSurface.hxx"
 #include "TIASurface.hxx"
 #include "FrameBuffer.hxx"
+#include "StateManager.hxx"
+#include "RewindManager.hxx"
 
 #ifdef DEBUGGER_SUPPORT
   #include "Debugger.hxx"
@@ -422,6 +424,44 @@ void FrameBuffer::update(bool force)
       break;  // EventHandlerState::TIMEMACHINE
     }
 
+    case EventHandlerState::PLAYBACK:
+    {
+      static Int32 frames = 0;
+      RewindManager& r = myOSystem.state().rewindManager();
+      bool success = true;
+      Int64 frameCycles = 76 * std::max<Int32>(myOSystem.console().tia().scanlinesLastFrame(), 240);
+
+      if(--frames <= 0)
+      {
+        r.unwindStates(1);
+        // get time between current and next state
+        uInt64 startCycles = r.getCurrentCycles();
+        success = r.unwindStates(1);
+        // display larger state gaps faster
+        frames = std::sqrt((myOSystem.console().tia().cycles() - startCycles) / frameCycles);
+
+        if(success)
+          r.rewindStates(1);
+      }
+
+      force = force || success;
+      if (force)
+        myTIASurface->render();
+
+      // Stop playback mode at the end of the state buffer
+      // and switch to Time Machine or Pause mode
+      if (!success)
+      {
+        frames = 0;
+      #ifdef GUI_SUPPORT
+        myOSystem.eventHandler().enterMenuMode(EventHandlerState::TIMEMACHINE);
+      #else
+        myOSystem.eventHandler().changeStateByEvent(Event::TogglePauseMode);
+      #endif
+      }
+      break;  // EventHandlerState::PLAYBACK
+    }
+
     case EventHandlerState::LAUNCHER:
     {
       force = force || myOSystem.launcher().needsRedraw();
@@ -467,8 +507,6 @@ void FrameBuffer::update(bool force)
 void FrameBuffer::updateInEmulationMode(float framesPerSecond)
 {
   // Update method that is specifically tailored to emulation mode
-  // Typically called from a thread, so it needs to be separate from
-  // the normal update() method
   //
   // We don't worry about selective rendering here; the rendering
   // always happens at the full framerate
@@ -634,11 +672,15 @@ void FrameBuffer::drawFrameStats(float framesPerSecond)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBuffer::toggleFrameStats()
+void FrameBuffer::toggleFrameStats(bool toggle)
 {
-  showFrameStats(!myStatsEnabled);
+  if (toggle)
+    showFrameStats(!myStatsEnabled);
   myOSystem.settings().setValue(
     myOSystem.settings().getBool("dev.settings") ? "dev.stats" : "plr.stats", myStatsEnabled);
+
+  myOSystem.frameBuffer().showMessage(string("Console info ") +
+                                      (myStatsEnabled ? "enabled" : "disabled"));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -793,7 +835,8 @@ void FrameBuffer::setPauseDelay()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-shared_ptr<FBSurface> FrameBuffer::allocateSurface(int w, int h, ScalingInterpolation interpolation, const uInt32* data)
+shared_ptr<FBSurface> FrameBuffer::allocateSurface(int w, int h, ScalingInterpolation interpolation,
+                                                   const uInt32* data)
 {
   // Add new surface to the list
   mySurfaceList.push_back(createSurface(w, h, interpolation, data));
@@ -862,6 +905,7 @@ void FrameBuffer::setUIPalette()
   const UIPaletteArray& ui_palette =
      (myOSystem.settings().getString("uipalette") == "classic") ? ourClassicUIPalette :
      (myOSystem.settings().getString("uipalette") == "light")   ? ourLightUIPalette :
+     (myOSystem.settings().getString("uipalette") == "dark")    ? ourDarkUIPalette :
       ourStandardUIPalette;
 
   for(size_t i = 0, j = myFullPalette.size() - ui_palette.size();
@@ -991,21 +1035,33 @@ void FrameBuffer::setFullscreen(bool enable)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FrameBuffer::toggleFullscreen(bool toggle)
 {
-  const bool isFullscreen = toggle ? !fullScreen() : fullScreen();
-
-  setFullscreen(isFullscreen);
-
-  if(myBufferType == BufferType::Emulator)
+  switch (myOSystem.eventHandler().state())
   {
-    ostringstream msg;
+    case EventHandlerState::LAUNCHER:
+    case EventHandlerState::EMULATION:
+    case EventHandlerState::PAUSE:
+    case EventHandlerState::DEBUGGER:
+    {
+      const bool isFullscreen = toggle ? !fullScreen() : fullScreen();
 
-    msg << "Fullscreen ";
-    if(isFullscreen)
-      msg << "enabled (" << refreshRate() << " Hz)";
-    else
-      msg << "disabled";
+      setFullscreen(isFullscreen);
 
-    showMessage(msg.str());
+      if (myBufferType != BufferType::Launcher)
+      {
+        ostringstream msg;
+
+        msg << "Fullscreen ";
+        if (isFullscreen)
+          msg << "enabled (" << refreshRate() << " Hz)";
+        else
+          msg << "disabled";
+
+        showMessage(msg.str());
+      }
+      break;
+    }
+    default:
+      break;
   }
 }
 
@@ -1553,3 +1609,16 @@ UIPaletteArray FrameBuffer::ourLightUIPalette = {
     0xffffff, 0x333333, 0xf0f0f0, 0x808080, 0xc0c0c0            // other
   }
 };
+
+UIPaletteArray FrameBuffer::ourDarkUIPalette = {
+  { 0x646464, 0xc0c0c0, 0x3c3c3c, 0x282828, 0x989898,           // base
+    0xc0c0c0, 0x1567a5, 0x0059a3, 0xc0c0c0,                     // text
+    0x202020, 0x000000, 0x0059a3, 0xb0b0b0,                     // UI elements
+    0x282828, 0x00467f, 0x646464, 0x0059a3, 0xc0c0c0, 0xc0c0c0, // buttons
+    0x989898,                                                   // checkbox
+    0x3c3c3c, 0x646464,                                         // scrollbar
+    0x7f2020, 0xc0c0c0, 0xe00000, 0xc00000,                     // debugger
+    0x989898, 0x0059a3, 0x3c3c3c, 0x000000, 0x3c3c3c,           // slider
+    0x000000, 0x989898, 0x202020, 0x646464, 0x3c3c3c            // other
+  }
+};
