@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2019 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -20,9 +20,9 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Keyboard::Keyboard(Jack jack, const Event& event, const System& system)
-  : Controller(jack, event, system, Controller::Keyboard)
+  : Controller(jack, event, system, Controller::Type::Keyboard)
 {
-  if(myJack == Left)
+  if(myJack == Jack::Left)
   {
     myOneEvent   = Event::KeyboardZero1;
     myTwoEvent   = Event::KeyboardZero2;
@@ -57,43 +57,41 @@ Keyboard::Keyboard(Jack jack, const Event& event, const System& system)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Keyboard::write(DigitalPin pin, bool value)
 {
-  myDigitalPinState[pin] = value;
+  setPin(pin, value);
 
   // Set defaults
-  myDigitalPinState[Six] = true;
+  setPin(DigitalPin::Six, true);
   Int32 resistanceFive = MIN_RESISTANCE;
   Int32 resistanceNine = MIN_RESISTANCE;
 
   // Now scan the rows and columns
-  if(!myDigitalPinState[Four])
+  if(!getPin(DigitalPin::Four))
   {
-    myDigitalPinState[Six] = (myEvent.get(myPoundEvent) == 0);
+    setPin(DigitalPin::Six, myEvent.get(myPoundEvent) == 0);
     if(myEvent.get(myZeroEvent) != 0) resistanceFive = MAX_RESISTANCE;
     if(myEvent.get(myStarEvent) != 0) resistanceNine = MAX_RESISTANCE;
   }
-  if(!myDigitalPinState[Three])
+  if(!getPin(DigitalPin::Three))
   {
-    myDigitalPinState[Six] = (myEvent.get(myNineEvent) == 0);
+    setPin(DigitalPin::Six, myEvent.get(myNineEvent) == 0);
     if(myEvent.get(myEightEvent) != 0) resistanceFive = MAX_RESISTANCE;
     if(myEvent.get(mySevenEvent) != 0) resistanceNine = MAX_RESISTANCE;
   }
-  if(!myDigitalPinState[Two])
+  if(!getPin(DigitalPin::Two))
   {
-    myDigitalPinState[Six] = (myEvent.get(mySixEvent) == 0);
+    setPin(DigitalPin::Six, myEvent.get(mySixEvent) == 0);
     if(myEvent.get(myFiveEvent) != 0) resistanceFive = MAX_RESISTANCE;
     if(myEvent.get(myFourEvent) != 0) resistanceNine = MAX_RESISTANCE;
   }
-  if(!myDigitalPinState[One])
+  if(!getPin(DigitalPin::One))
   {
-    myDigitalPinState[Six] = (myEvent.get(myThreeEvent) == 0);
+    setPin(DigitalPin::Six, myEvent.get(myThreeEvent) == 0);
     if(myEvent.get(myTwoEvent) != 0) resistanceFive = MAX_RESISTANCE;
     if(myEvent.get(myOneEvent) != 0) resistanceNine = MAX_RESISTANCE;
   }
 
-  if (resistanceFive != read(Five)) {
-    updateAnalogPin(Five, resistanceFive);
-  }
-
-  if (resistanceNine != read(Nine))
-    updateAnalogPin(Nine, resistanceNine);
+  if(resistanceFive != read(AnalogPin::Five))
+    setPin(AnalogPin::Five, resistanceFive);
+  if(resistanceNine != read(AnalogPin::Nine))
+    setPin(AnalogPin::Nine, resistanceNine);
 }
