@@ -19,37 +19,55 @@
 
 #include "bspf.hxx"
 #include "FSNode.hxx"
+#include "Logger.hxx"
 #include "DefProps.hxx"
 #include "Props.hxx"
 #include "PropsSet.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PropertiesSet::load(const string& filename, bool save)
+void PropertiesSet::load(const FilesystemNode& file, bool save)
 {
-  ifstream in(filename);
-
-  Properties prop;
-  while(in >> prop)
-    insert(prop, save);
+  try
+  {
+    stringstream in;
+    if(file.exists() && file.read(in) > 0)
+    {
+      Properties prop;
+      while(in >> prop)
+        insert(prop, save);
+    }
+  }
+  catch(...)
+  {
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool PropertiesSet::save(const string& filename) const
+bool PropertiesSet::save(const FilesystemNode& file) const
 {
-  // Only save properties when it won't create an empty file
-  FilesystemNode props(filename);
-  if(!props.exists() && myExternalProps.size() == 0)
-    return false;
+  try
+  {
+    // Only save properties when it won't create an empty file
+    if(!file.exists() && myExternalProps.size() == 0)
+      return false;
 
-  ofstream out(filename);
-  if(!out)
-    return false;
+    // Only save those entries in the external list
+    stringstream out;
+    for(const auto& i: myExternalProps)
+      out << i.second;
 
-  // Only save those entries in the external list
-  for(const auto& i: myExternalProps)
-    out << i.second;
+    file.write(out);
+    return true;
+  }
+  catch(const runtime_error& e)
+  {
+    Logger::error(e.what());
+  }
+  catch(...)
+  {
+  }
 
-  return true;
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -166,8 +184,8 @@ void PropertiesSet::loadPerROM(const FilesystemNode& rom, const string& md5)
   // First, does this ROM have a per-ROM properties entry?
   // If so, load it into the database
   FilesystemNode propsNode(rom.getPathWithExt(".pro"));
-  if(propsNode.exists() && propsNode.isFile())
-    load(propsNode.getPath(), false);
+  if(propsNode.exists())
+    load(propsNode, false);
 
   // Next, make sure we have a valid md5 and name
   Properties props;
