@@ -38,6 +38,7 @@ Settings::Settings()
 
   // If no version is recorded with the persisted settings, we set it to zero
   setPermanent(SETTINGS_VERSION_KEY, 0);
+  setPermanent("stella.version", "6.2.1");
 
   // Video-related options
   setPermanent("video", "");
@@ -56,6 +57,7 @@ Settings::Settings()
   setPermanent("tia.fs_overscan", "0");
   setPermanent("tia.vsizeadjust", 0);
   setPermanent("tia.dbgcolors", "roygpb");
+  setPermanent("tia.correct_aspect", "true");
   // Palette options
   setPermanent("palette", PaletteHandler::SETTING_STANDARD);
   setPermanent("pal.phase_ntsc", "26.2");
@@ -80,6 +82,7 @@ Settings::Settings()
   // Sound options
   setPermanent(AudioSettings::SETTING_ENABLED, AudioSettings::DEFAULT_ENABLED);
   setPermanent(AudioSettings::SETTING_VOLUME, AudioSettings::DEFAULT_VOLUME);
+  setPermanent(AudioSettings::SETTING_DEVICE, AudioSettings::DEFAULT_DEVICE);
   setPermanent(AudioSettings::SETTING_PRESET, static_cast<int>(AudioSettings::DEFAULT_PRESET));
   setPermanent(AudioSettings::SETTING_FRAGMENT_SIZE, AudioSettings::DEFAULT_FRAGMENT_SIZE);
   setPermanent(AudioSettings::SETTING_SAMPLE_RATE, AudioSettings::DEFAULT_SAMPLE_RATE);
@@ -100,6 +103,7 @@ Settings::Settings()
   setPermanent("combomap", "");
   setPermanent("joydeadzone", "13");
   setPermanent("joyallow4", "false");
+  setPermanent("autofirerate", "0");
   setPermanent("usemouse", "analog");
   setPermanent("grabmouse", "true");
   setPermanent("cursor", "2");
@@ -196,6 +200,7 @@ Settings::Settings()
   setPermanent("plr.tm.uncompressed", 60);
   setPermanent("plr.tm.interval", "30f"); // = 0.5 seconds
   setPermanent("plr.tm.horizon", "10m"); // = ~10 minutes
+  setPermanent("plr.detectedinfo", "false");
   setPermanent("plr.eepromaccess", "false");
 
   // Developer settings
@@ -216,6 +221,7 @@ Settings::Settings()
   setPermanent("dev.tia.blinvphase", "true");
   setPermanent("dev.tia.delaypfbits", "true");
   setPermanent("dev.tia.delaypfcolor", "true");
+  setPermanent("dev.tia.delaybkcolor", "true");
   setPermanent("dev.tia.delayplswap", "true");
   setPermanent("dev.tia.delayblswap", "true");
   setPermanent("dev.timemachine", true);
@@ -225,6 +231,7 @@ Settings::Settings()
   setPermanent("dev.tm.horizon", "30s"); // = ~30 seconds
   // Thumb ARM emulation options
   setPermanent("dev.thumb.trapfatal", "true");
+  setPermanent("dev.detectedinfo", "true");
   setPermanent("dev.eepromaccess", "true");
 }
 
@@ -425,6 +432,7 @@ void Settings::usage() const
   #ifdef SOUND_SUPPORT
     << "  -audio.enabled            <1|0>      Enable audio\n"
     << "  -audio.volume             <0-100>    Volume\n"
+    << "  -audio.device             <number>   ID of the audio device (0 = default)\n"
     << "  -audio.preset             <1-5>      Audio preset (or 1 for custom)\n"
     << "  -audio.sample_rate        <number>   Output sample rate (44100|48000|96000)\n"
     << "  -audio.fragment_size      <number>   Fragment size (128|256|512|1024|\n"
@@ -446,6 +454,7 @@ void Settings::usage() const
     << "  -tia.fs_overscan <0-10>       Add overscan to TIA image in fullscreen mode\n"
     << "  -tia.dbgcolors   <string>     Debug colors to use for each object (see manual\n"
     << "                                 for description)\n"
+    << "  -tia.correct_aspect <1|0>     Enable aspect ratio correct scaling\n"
     << endl
     << "  -tv.filter    <0-5>           Set TV effects off (0) or to specified mode\n"
     << "                                 (1-5)\n"
@@ -481,6 +490,7 @@ void Settings::usage() const
     << "  -tsense       <1-20>         Sensitivity of mouse emulated trackball movement\n"
     << "  -dcsense      <1-20>         Sensitivity of digital emulated driving controller\n"
     << "                                movement\n"
+    << "  -autofirerate <0-30>         Set fire button's autofire rate (0 means off)\n"
     << "  -saport       <lr|rl>        How to assign virtual ports to multiple\n"
     << "                                Stelladaptor/2600-daptors\n"
     << "  -modcombo     <1|0>          Enable modifer key combos\n"
@@ -524,7 +534,8 @@ void Settings::usage() const
     << "                                launcher\n"
     << "  -romloadcount <number>       Number of ROM to load next from multicard\n"
     << "  -uipalette    <standard|     Selects GUI theme\n"
-    << "                 classic|light>\n"
+    << "                 classic|\n"
+    << "                 light|dark>\n"
     << "  -hidpi        <0|1>          Enable HiDPI mode\n"
     << "  -dialogfont   <small|        Use the specified font in the dialogs\n"
     << "                 low_medium|\n"
@@ -611,6 +622,7 @@ void Settings::usage() const
     << "                                    mode\n"
     << endl
     << "  -plr.stats        <1|0>          Overlay console info during emulation\n"
+    << "  -plr.detectedinfo <1|0>          Enable initial detected settings info\n"
     << "  -plr.console      <2600|7800>    Select console for B/W and Pause key\n"
     << "                                    handling and RAM initialization\n"
     << "  -plr.bankrandom   <1|0>          Randomize the startup bank on reset\n"
@@ -626,6 +638,7 @@ void Settings::usage() const
     << endl
     << " The same parameters but for developer settings mode\n"
     << "  -dev.stats        <1|0>          Overlay console info during emulation\n"
+    << "  -dev.detectedinfo <1|0>          Enable initial detected settings info\n"
     << "  -dev.console      <2600|7800>    Select console for B/W and Pause key\n"
     << "                                    handling and RAM initialization\n"
     << "  -dev.bankrandom   <1|0>          Randomize the startup bank on reset\n"
@@ -649,13 +662,15 @@ void Settings::usage() const
     << "  -dev.tia.type <standard|custom|  Selects a TIA type\n"
     << "                 koolaidman|\n"
     << "                 cosmicark|pesco|\n"
-    << "                 quickstep|heman|>\n"
+    << "                 quickstep|\n"
+    << "                 indy500|heman|>\n"
     << "  -dev.tia.plinvphase   <1|0>      Enable inverted HMOVE clock phase for players\n"
     << "  -dev.tia.msinvphase   <1|0>      Enable inverted HMOVE clock phase for\n"
     << "                                    missiles\n"
     << "  -dev.tia.blinvphase   <1|0>      Enable inverted HMOVE clock phase for ball\n"
     << "  -dev.tia.delaypfbits  <1|0>      Enable extra delay cycle for PF bits access\n"
     << "  -dev.tia.delaypfcolor <1|0>      Enable extra delay cycle for PF color\n"
+    << "  -dev.tia.delaybkcolor <1|0>      Enable extra delay cycle for background color\n"
     << "  -dev.tia.delayplswap  <1|0>      Enable extra delay cycle for VDELP0/1 swap\n"
     << "  -dev.tia.delayblswap  <1|0>      Enable extra delay cycle for VDELBL swap\n"
     << endl << std::flush;
