@@ -56,7 +56,9 @@ TIASurface::TIASurface(OSystem& system)
   myTiaSurface = myFB.allocateSurface(
     AtariNTSC::outWidth(TIAConstants::frameBufferWidth),
     TIAConstants::frameBufferHeight,
-    interpolationModeFromSettings(myOSystem.settings())
+    !correctAspect()
+      ? FrameBuffer::ScalingInterpolation::none
+      : interpolationModeFromSettings(myOSystem.settings())
   );
 
   // Generate scanline data, and a pre-defined scanline surface
@@ -312,8 +314,8 @@ void TIASurface::enableNTSC(bool enable)
 string TIASurface::effectsInfo() const
 {
   const FBSurface::Attributes& attr = mySLineSurface->attributes();
-
   ostringstream buf;
+
   switch(myFilter)
   {
     case Filter::Normal:
@@ -326,12 +328,12 @@ string TIASurface::effectsInfo() const
       buf << myNTSCFilter.getPreset() << ", scanlines=" << attr.blendalpha;
       break;
     case Filter::BlarggPhosphor:
-      buf << myNTSCFilter.getPreset() << ", phosphor, scanlines="
-          << attr.blendalpha;
+      buf << myNTSCFilter.getPreset() << ", phosphor, scanlines=" << attr.blendalpha;
       break;
   }
 
   buf << ", inter=" << (myOSystem.settings().getBool("tia.inter") ? "enabled" : "disabled");
+  buf << ", aspect correction=" << (correctAspect() ? "enabled" : "disabled");
 
   return buf.str();
 }
@@ -515,3 +517,8 @@ void TIASurface::updateSurfaceSettings()
       interpolationModeFromSettings(myOSystem.settings())
   );
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool TIASurface::correctAspect() const {
+  return myOSystem.settings().getBool("tia.correct_aspect");
+}
