@@ -285,6 +285,7 @@ bool TIA::save(Serializer& out) const
 
     out.putByte(myPFBitsDelay);
     out.putByte(myPFColorDelay);
+    out.putByte(myBKColorDelay);
     out.putByte(myPlSwapDelay);
   }
   catch(...)
@@ -356,6 +357,7 @@ bool TIA::load(Serializer& in)
 
     myPFBitsDelay = in.getByte();
     myPFColorDelay = in.getByte();
+    myBKColorDelay = in.getByte();
     myPlSwapDelay = in.getByte();
 
     // Re-apply dev settings
@@ -521,6 +523,7 @@ bool TIA::poke(uInt16 address, uInt8 value)
 
       for (PaddleReader& paddleReader : myPaddleReaders)
         paddleReader.vblank(value, myTimestamp);
+      updateDumpPorts(value);
 
       myDelayQueue.push(VBLANK, value, Delay::vblank);
 
@@ -605,8 +608,13 @@ bool TIA::poke(uInt16 address, uInt8 value)
     case COLUBK:
     {
       value &= 0xFE;
-      myBackground.setColor(value);
-      myShadowRegisters[address] = value;
+      if(myBKColorDelay)
+        myDelayQueue.push(COLUBK, value, 1);
+      else
+      {
+        myBackground.setColor(value);
+        myShadowRegisters[address] = value;
+      }
     #ifdef DEBUGGER_SUPPORT
       uInt16 dataAddr = mySystem->m6502().lastDataAddressForPoke();
       if(dataAddr)
@@ -918,6 +926,9 @@ void TIA::applyDeveloperSettings()
     setPFColorDelay(custom
                     ? mySettings.getBool("dev.tia.delaypfcolor")
                     : BSPF::equalsIgnoreCase("quickstep", mySettings.getString("dev.tia.type")));
+    setBKColorDelay(custom
+                    ? mySettings.getBool("dev.tia.delaybkcolor")
+                    : BSPF::equalsIgnoreCase("indy500", mySettings.getString("dev.tia.type")));
     setPlSwapDelay(custom
                    ? mySettings.getBool("dev.tia.delayplswap")
                    : BSPF::equalsIgnoreCase("heman", mySettings.getString("dev.tia.type")));
@@ -930,6 +941,7 @@ void TIA::applyDeveloperSettings()
     setBlInvertedPhaseClock(false);
     setPFBitsDelay(false);
     setPFColorDelay(false);
+    setBKColorDelay(false);
     setPlSwapDelay(false);
     setBlSwapDelay(false);
   }
@@ -1029,9 +1041,13 @@ bool TIA::toggleBit(TIABit b, uInt8 mode)
       mask = b;
       break;
 
-    default:
+    case 2:
       mask = (~mySpriteEnabledBits & b);
       break;
+
+    default:
+      mask = (mySpriteEnabledBits & b);
+      break;
   }
 
   mySpriteEnabledBits = (mySpriteEnabledBits & ~b) | mask;
@@ -1047,9 +1063,11 @@ bool TIA::toggleBit(TIABit b, uInt8 mode)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool TIA::toggleBits()
+bool TIA::toggleBits(bool toggle)
 {
-  toggleBit(TIABit(0xFF), mySpriteEnabledBits > 0 ? 0 : 1);
+  toggleBit(TIABit(0xFF), toggle
+                          ? mySpriteEnabledBits > 0 ? 0 : 1
+                          : mySpriteEnabledBits);
 
   return mySpriteEnabledBits;
 }
@@ -1068,9 +1086,13 @@ bool TIA::toggleCollision(TIABit b, uInt8 mode)
       mask = b;
       break;
 
-    default:
+    case 2:
       mask = (~myCollisionsEnabledBits & b);
       break;
+
+    default:
+      mask = (myCollisionsEnabledBits & b);
+      break;
   }
 
   myCollisionsEnabledBits = (myCollisionsEnabledBits & ~b) | mask;
@@ -1086,9 +1108,11 @@ bool TIA::toggleCollision(TIABit b, uInt8 mode)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool TIA::toggleCollisions()
+bool TIA::toggleCollisions(bool toggle)
 {
-  toggleCollision(TIABit(0xFF), myCollisionsEnabledBits > 0 ? 0 : 1);
+  toggleCollision(TIABit(0xFF), toggle
+                                ? myCollisionsEnabledBits > 0 ? 0 : 1
+                                : myCollisionsEnabledBits);
 
   return myCollisionsEnabledBits;
 }
@@ -1207,6 +1231,9 @@ bool TIA::toggleJitter(uInt8 mode)
       myEnableJitter = !myEnableJitter;
       break;
 
+    case 3:
+      break;
+
     default:
       throw runtime_error("invalid argument for toggleJitter");
   }
@@ -1593,6 +1620,12 @@ void TIA::setPFColorDelay(bool delayed)
   myPFColorDelay = delayed ? 1 : 0;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::setBKColorDelay(bool delayed)
+{
+  myBKColorDelay = delayed ? 1 : 0;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::setPlSwapDelay(bool delayed)
 {
@@ -1668,6 +1701,10 @@ void TIA::delayedWrite(uInt8 address, uInt8 value)
       myPlayfield.pf2(value);
       break;
 
+    case COLUBK:
+      myBackground.setColor(value);
+      break;
+
     case COLUPF:
       myPlayfield.setColor(value);
       myBall.setColor(value);
@@ -1945,6 +1982,24 @@ void TIA::toggleCollBLPF()
   myCollisionMask ^= (CollisionMask::ball & CollisionMask::playfield);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void TIA::updateDumpPorts(uInt8 value)
+{
+  bool newIsDumped = value & 0x80;
+
+  if(myArePortsDumped != newIsDumped)
+  {
+    myArePortsDumped = newIsDumped;
+    myDumpPortsTimestamp = myTimestamp;
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Int64 TIA::dumpPortsCycles()
+{
+  return (myTimestamp - myDumpPortsTimestamp) / 3;
+}
+
 #ifdef DEBUGGER_SUPPORT
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void TIA::createAccessArrays()
