@@ -31,6 +31,7 @@
 #include "PopUpWidget.hxx"
 #include "Props.hxx"
 #include "PropsSet.hxx"
+#include "QuadTariDialog.hxx"
 #include "TabWidget.hxx"
 #include "TIAConstants.hxx"
 #include "Widget.hxx"
@@ -227,6 +228,7 @@ GameInfoDialog::GameInfoDialog(
   VarList::push_back(ctrls, "KidVid", "KIDVID");
   VarList::push_back(ctrls, "Lightgun", "LIGHTGUN");
   VarList::push_back(ctrls, "MindLink", "MINDLINK");
+  VarList::push_back(ctrls, "QuadTari", "QUADTARI");
 
   ypos = VBORDER;
   pwidth = font.getStringWidth("Paddles_IAxis");
@@ -246,20 +248,21 @@ GameInfoDialog::GameInfoDialog(
                                 myRightPortLabel->getTop()-1,
                                 pwidth, lineHeight, ctrls, "", 0, kRightCChanged);
   wid.push_back(myRightPort);
+
   ypos += lineHeight + VGAP;
   myRightPortDetected = new StaticTextWidget(myTab, ifont, myRightPort->getLeft(), ypos,
                                              "Sega Genesis detected");
-  ypos += ifont.getLineHeight() + VGAP + 4;
 
-  mySwapPorts = new CheckboxWidget(myTab, font, myLeftPort->getRight() + fontWidth*4,
-                                   myLeftPort->getTop()+1, "Swap ports");
+  mySwapPorts = new CheckboxWidget(myTab, font, myLeftPort->getRight() + fontWidth * 4,
+                                   myLeftPort->getTop() + 1, "Swap ports");
   wid.push_back(mySwapPorts);
-  mySwapPaddles = new CheckboxWidget(myTab, font, myRightPort->getRight() + fontWidth*4,
-                                     myRightPort->getTop()+1, "Swap paddles");
-  wid.push_back(mySwapPaddles);
+
+  myQuadTariButton = new ButtonWidget(myTab, font, myRightPort->getRight() + fontWidth * 4, myRightPort->getTop() - 2,
+                                      " QuadTari" + ELLIPSIS + " ", kQuadTariPressed);
+  wid.push_back(myQuadTariButton);
 
   // EEPROM erase button for left/right controller
-  //ypos += lineHeight + VGAP + 4;
+  ypos += ifont.getLineHeight() + VGAP + 4;
   pwidth = myRightPort->getWidth();   //font.getStringWidth("Erase EEPROM ") + 23;
   myEraseEEPROMLabel = new StaticTextWidget(myTab, font, HBORDER, ypos, "AtariVox/SaveKey ");
   myEraseEEPROMButton = new ButtonWidget(myTab, font, myEraseEEPROMLabel->getRight(), ypos - 4,
@@ -270,6 +273,10 @@ GameInfoDialog::GameInfoDialog(
                                            "(for this game only)");
   ypos += lineHeight + VGAP * 4;
 
+  mySwapPaddles = new CheckboxWidget(myTab, font, xpos, ypos, "Swap paddles");
+  wid.push_back(mySwapPaddles);
+  ypos += lineHeight + VGAP;
+
   // Paddles
   myPaddlesCenter = new StaticTextWidget(myTab, font, xpos, ypos, "Paddles center:");
   ypos += lineHeight + VGAP;
@@ -292,7 +299,7 @@ GameInfoDialog::GameInfoDialog(
 
   // Mouse
   xpos = HBORDER + fontWidth * 24 - INDENT;
-  ypos = myPaddlesCenter->getTop();
+  ypos = mySwapPaddles->getTop() - 1;
   myMouseControl = new CheckboxWidget(myTab, font, xpos, ypos + 1, "Specific mouse axes",
                                       kMCtrlChanged);
   wid.push_back(myMouseControl);
@@ -387,6 +394,11 @@ GameInfoDialog::GameInfoDialog(
   addBGroupToFocusList(wid);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+GameInfoDialog::~GameInfoDialog()
+{
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void GameInfoDialog::loadConfig()
 {
@@ -590,8 +602,22 @@ void GameInfoDialog::saveProperties()
   myGameProperties.set(PropType::Console_RightDiff, myRightDiffGroup->getSelected() ? "B" : "A");
 
   // Controller properties
-  myGameProperties.set(PropType::Controller_Left, myLeftPort->getSelectedTag().toString());
-  myGameProperties.set(PropType::Controller_Right, myRightPort->getSelectedTag().toString());
+  string controller = myLeftPort->getSelectedTag().toString();
+  myGameProperties.set(PropType::Controller_Left, controller);
+  if(controller != "AUTO" && controller != "QUADTARI")
+  {
+    myGameProperties.set(PropType::Controller_Left1, "");
+    myGameProperties.set(PropType::Controller_Left2, "");
+  }
+
+  controller = myRightPort->getSelectedTag().toString();
+  myGameProperties.set(PropType::Controller_Right, controller);
+  if(controller != "AUTO" && controller != "QUADTARI")
+  {
+    myGameProperties.set(PropType::Controller_Right1, "");
+    myGameProperties.set(PropType::Controller_Right2, "");
+  }
+
   myGameProperties.set(PropType::Console_SwapPorts, (mySwapPorts->isEnabled() && mySwapPorts->getState()) ? "YES" : "NO");
   myGameProperties.set(PropType::Controller_SwapPaddles, mySwapPaddles->getState() ? "YES" : "NO");
 
@@ -704,10 +730,14 @@ void GameInfoDialog::updateControllerStates()
   if(type == Controller::Type::Unknown)
   {
     if(instance().hasConsole())
+    {
       label = (!swapPorts ? instance().console().leftController().name()
                : instance().console().rightController().name()) + " detected";
+      if(BSPF::startsWithIgnoreCase(label, "QUADTARI"))
+        label = "QuadTari detected"; // remove plugged-in controller names
+    }
     else if(autoDetect)
-      label = ControllerDetector::detectName(image.get(), size, type,
+      label = ControllerDetector::detectName(image, size, type,
                                              !swapPorts ? Controller::Jack::Left : Controller::Jack::Right,
                                              instance().settings()) + " detected";
   }
@@ -719,10 +749,14 @@ void GameInfoDialog::updateControllerStates()
   if(type == Controller::Type::Unknown)
   {
     if(instance().hasConsole())
+    {
       label = (!swapPorts ? instance().console().rightController().name()
                : instance().console().leftController().name()) + " detected";
+      if(BSPF::startsWithIgnoreCase(label, "QUADTARI"))
+        label = "QuadTari detected"; // remove plugged-in controller names
+    }
     else if(autoDetect)
-      label = ControllerDetector::detectName(image.get(), size, type,
+      label = ControllerDetector::detectName(image, size, type,
                                              !swapPorts ? Controller::Jack::Right : Controller::Jack::Left,
                                              instance().settings()) + " detected";
   }
@@ -759,6 +793,10 @@ void GameInfoDialog::updateControllerStates()
   myRightPortLabel->setEnabled(enableSelectControl);
   myLeftPort->setEnabled(enableSelectControl);
   myRightPort->setEnabled(enableSelectControl);
+  myQuadTariButton->setEnabled(BSPF::startsWithIgnoreCase(contrLeft, "QUADTARI") ||
+                               BSPF::startsWithIgnoreCase(contrRight, "QUADTARI") ||
+                               BSPF::startsWithIgnoreCase(myLeftPortDetected->getLabel(), "QUADTARI") ||
+                               BSPF::startsWithIgnoreCase(myRightPortDetected->getLabel(), "QUADTARI"));
 
   mySwapPorts->setEnabled(enableSelectControl);
   mySwapPaddles->setEnabled(enablePaddles);
@@ -771,7 +809,6 @@ void GameInfoDialog::updateControllerStates()
   myPaddleXCenter->setEnabled(enablePaddles);
   myPaddleYCenter->setEnabled(enablePaddles);
 
-
   bool enableMouse = enablePaddles ||
     BSPF::startsWithIgnoreCase(contrLeft, "Driving") ||
     BSPF::startsWithIgnoreCase(contrRight, "Driving") ||
@@ -809,24 +846,29 @@ void GameInfoDialog::eraseEEPROM()
 void GameInfoDialog::saveCurrentPropertiesToDisk()
 {
   saveProperties();
+  stringstream out;
+  out << myGameProperties;
 
-  FilesystemNode propfile(instance().defaultSaveDir() + myGameFile.getNameWithExt(".pro"));
-  ofstream out(propfile.getPath());
-  if(out)
+  try
   {
-    out << myGameProperties;
+    FilesystemNode propfile = instance().defaultSaveDir();
+    propfile /= myGameFile.getNameWithExt(".pro");
+
+    propfile.write(out);
     instance().frameBuffer().showMessage("Properties saved to " +
                                          propfile.getShortPath());
   }
-  else
+  catch(...)
+  {
     instance().frameBuffer().showMessage("Error saving properties");
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void GameInfoDialog::handleCommand(CommandSender* sender, int cmd,
                                    int data, int id)
 {
-  switch (cmd)
+  switch(cmd)
   {
     case GuiObject::kOKCmd:
       saveConfig();
@@ -854,6 +896,22 @@ void GameInfoDialog::handleCommand(CommandSender* sender, int cmd,
       updateControllerStates();
       break;
 
+    case kQuadTariPressed:
+    {
+      bool enableLeft =
+        BSPF::startsWithIgnoreCase(myLeftPort->getSelectedTag().toString(), "QUADTARI") ||
+        BSPF::startsWithIgnoreCase(myLeftPortDetected->getLabel(), "QUADTARI");
+      bool enableRight =
+        BSPF::startsWithIgnoreCase(myRightPort->getSelectedTag().toString(), "QUADTARI") ||
+        BSPF::startsWithIgnoreCase(myRightPortDetected->getLabel(), "QUADTARI");
+
+      if(!myQuadTariDialog)
+        myQuadTariDialog = make_unique<QuadTariDialog>
+          (this, _font, _font.getMaxCharWidth() * 37, _font.getFontHeight() * 8,
+           myGameProperties);
+      myQuadTariDialog->show(enableLeft, enableRight);
+      break;
+    }
     case kEEButtonPressed:
       eraseEEPROM();
       break;
