@@ -85,6 +85,7 @@ UIDialog::UIDialog(OSystem& osystem, DialogContainer& parent,
   VarList::push_back(items, "Standard", "standard");
   VarList::push_back(items, "Classic", "classic");
   VarList::push_back(items, "Light", "light");
+  VarList::push_back(items, "Dark", "dark");
   myPalettePopup = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
                                    items, "Theme      ", lwidth);
   wid.push_back(myPalettePopup);
@@ -506,7 +507,7 @@ void UIDialog::setDefaults()
       myLauncherHeightSlider->setValue(h);
       myLauncherFontPopup->setSelected("medium", "");
       myRomViewerSize->setValue(35);
-      mySnapLoadPath->setText(instance().defaultLoadDir());
+      mySnapLoadPath->setText(instance().defaultLoadDir().getShortPath());
       myLauncherExitWidget->setState(false);
       break;
     }
@@ -521,12 +522,15 @@ void UIDialog::handleCommand(CommandSender* sender, int cmd, int data, int id)
   switch(cmd)
   {
     case GuiObject::kOKCmd:
+    {
+      bool inform = myIsGlobal &&
+        myRomPath->getText() != instance().settings().getString("romdir");
       saveConfig();
       close();
-      if(myIsGlobal)  // Let the boss know romdir has changed
+      if(inform) // Let the boss know romdir has changed
         sendCommand(LauncherDialog::kRomDirChosenCmd, 0, 0);
       break;
-
+    }
     case GuiObject::kDefaultsCmd:
       setDefaults();
       break;
