@@ -68,7 +68,7 @@ VideoAudioDialog::VideoAudioDialog(OSystem& osystem, DialogContainer& parent,
 
   // Set real dimensions
   setSize(44 * fontWidth + HBORDER * 2 + PopUpWidget::dropDownWidth(font) * 2,
-          _th + VGAP * 6 + lineHeight + 10 * (lineHeight + VGAP) + buttonHeight + VBORDER * 3,
+          _th + VGAP * 3 + lineHeight + 11 * (lineHeight + VGAP) + buttonHeight + VBORDER * 3,
           max_w, max_h);
 
   // The tab widget
@@ -102,6 +102,7 @@ VideoAudioDialog::VideoAudioDialog(OSystem& osystem, DialogContainer& parent,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void VideoAudioDialog::addDisplayTab()
 {
+  const GUI::Font& ifont = instance().frameBuffer().infoFont();
   const int lineHeight   = _font.getLineHeight(),
             fontHeight   = _font.getFontHeight(),
             fontWidth    = _font.getMaxCharWidth();
@@ -127,8 +128,9 @@ void VideoAudioDialog::addDisplayTab()
 
   // TIA interpolation
   myTIAInterpolate = new CheckboxWidget(myTab, _font, xpos, ypos + 1, "Interpolation ");
-  wid.push_back(myTIAInterpolate);  ypos += lineHeight + VGAP * 4;
+  wid.push_back(myTIAInterpolate);
 
+  ypos += lineHeight + VGAP * 4;
   // TIA zoom levels (will be dynamically filled later)
   myTIAZoom = new SliderWidget(myTab, _font, xpos, ypos - 1, swidth, lineHeight,
                                "Zoom ", lwidth, 0, fontWidth * 4, "%");
@@ -162,6 +164,11 @@ void VideoAudioDialog::addDisplayTab()
   myTVOverscan->setTickmarkIntervals(2);
   wid.push_back(myTVOverscan);
 
+  // Aspect ratio correction
+  ypos += lineHeight + VGAP * 4;
+  myCorrectAspect = new CheckboxWidget(myTab, _font, xpos, ypos + 1, "Correct aspect ratio (*)");
+  wid.push_back(myUseStretch);
+
   // Vertical size
   ypos += lineHeight + VGAP;
   myVSizeAdjust =
@@ -172,6 +179,13 @@ void VideoAudioDialog::addDisplayTab()
   wid.push_back(myVSizeAdjust);
 
 
+  // Add message concerning usage
+  ypos = myTab->getHeight() - fontHeight - ifont.getFontHeight() - VGAP - VBORDER;
+  int iwidth = ifont.getStringWidth("(*) Change may require an application restart");
+  new StaticTextWidget(myTab, ifont, xpos, ypos,
+                       std::min(iwidth, _w - HBORDER * 2), ifont.getFontHeight(),
+                       "(*) Change may require an application restart");
+
   // Add items for tab 0
   addToFocusList(wid, myTab, tabID);
 }
@@ -365,6 +379,14 @@ void VideoAudioDialog::addAudioTab()
   wid.push_back(myVolumeSlider);
   ypos += lineHeight + VGAP;
 
+  // Device
+  myDevicePopup = new PopUpWidget(myTab, _font, xpos, ypos,
+                                  _w - xpos - lwidth - HBORDER - PopUpWidget::dropDownWidth(_font) - 2, lineHeight,
+                                  instance().sound().supportedDevices(),
+                                  "Device", lwidth, kDeviceChanged);
+  wid.push_back(myDevicePopup);
+  ypos += lineHeight + VGAP;
+
   // Mode
   items.clear();
   VarList::push_back(items, "Low quality, medium lag", static_cast<int>(AudioSettings::Preset::lowQualityMediumLag));
@@ -486,6 +508,9 @@ void VideoAudioDialog::loadConfig()
   myTVOverscan->setValue(instance().settings().getInt("tia.fs_overscan"));
   handleFullScreenChange();
 
+  // Aspect ratio correction
+  myCorrectAspect->setState(instance().settings().getBool("tia.correct_aspect"));
+
   // Aspect ratio setting (NTSC and PAL)
   myVSizeAdjust->setValue(instance().settings().getInt("tia.vsizeadjust"));
 
@@ -542,6 +567,11 @@ void VideoAudioDialog::loadConfig()
   // Volume
   myVolumeSlider->setValue(audioSettings.volume());
 
+  // Device
+  uInt32 deviceId = BSPF::clamp(audioSettings.device(), 0u,
+                                uInt32(instance().sound().supportedDevices().size() - 1));
+  myDevicePopup->setSelected(deviceId);
+
   // Stereo
   myStereoSoundCheckbox->setState(audioSettings.stereo());
 
@@ -603,6 +633,9 @@ void VideoAudioDialog::saveConfig()
   // TIA zoom levels
   instance().settings().setValue("tia.zoom", myTIAZoom->getValue() / 100.0);
 
+  // Aspect ratio correction
+  instance().settings().setValue("tia.correct_aspect", myCorrectAspect->getState());
+
   // Aspect ratio setting (NTSC and PAL)
   const int oldAdjust = instance().settings().getInt("tia.vsizeadjust");
   const int newAdjust = myVSizeAdjust->getValue();
@@ -666,6 +699,9 @@ void VideoAudioDialog::saveConfig()
   audioSettings.setVolume(myVolumeSlider->getValue());
   instance().sound().setVolume(myVolumeSlider->getValue());
 
+  // Device
+  audioSettings.setDevice(myDevicePopup->getSelected());
+
   // Stereo
   audioSettings.setStereo(myStereoSoundCheckbox->getState());
 
@@ -713,6 +749,7 @@ void VideoAudioDialog::setDefaults()
     #endif
       myTVOverscan->setValue(0);
       myTIAZoom->setValue(300);
+      myCorrectAspect->setState(true);
       myVSizeAdjust->setValue(0);
 
       handleFullScreenChange();
@@ -754,6 +791,7 @@ void VideoAudioDialog::setDefaults()
     case 3:  // Audio
       mySoundEnableCheckbox->setState(AudioSettings::DEFAULT_ENABLED);
       myVolumeSlider->setValue(AudioSettings::DEFAULT_VOLUME);
+      myDevicePopup->setSelected(AudioSettings::DEFAULT_DEVICE);
       myStereoSoundCheckbox->setState(AudioSettings::DEFAULT_STEREO);
       myDpcPitch->setValue(AudioSettings::DEFAULT_DPC_PITCH);
       myModePopup->setSelected(static_cast<int>(AudioSettings::DEFAULT_PRESET));
@@ -1061,6 +1099,7 @@ void VideoAudioDialog::updateEnabledState()
   bool userMode = preset == AudioSettings::Preset::custom;
 
   myVolumeSlider->setEnabled(active);
+  myDevicePopup->setEnabled(active);
   myStereoSoundCheckbox->setEnabled(active);
   myModePopup->setEnabled(active);
   // enable only for Pitfall II cart
