@@ -228,6 +228,18 @@ CpuMethod getCpuSpecial(char* ch)
     return nullptr;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// special methods that get RIOT internal state
+RiotMethod getRiotSpecial(char* ch)
+{
+  if(BSPF::equalsIgnoreCase(ch, "_timwrapread"))
+    return &RiotDebug::timWrappedOnRead;
+  else if(BSPF::equalsIgnoreCase(ch, "_timwrapwrite"))
+    return &RiotDebug::timWrappedOnWrite;
+  else
+    return nullptr;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // special methods that get TIA internal state
 TiaMethod getTiaSpecial(char* ch)
@@ -281,6 +293,7 @@ int yylex() {
         {
           CartMethod cartMeth;
           CpuMethod  cpuMeth;
+          RiotMethod riotMeth;
           TiaMethod  tiaMeth;
 
           char *bufp = idbuf;
@@ -309,6 +322,9 @@ int yylex() {
           } else if( (cartMeth = getCartSpecial(idbuf)) ) {
             yylval.cartMethod = cartMeth;
             return CART_METHOD;
+          } else if( (riotMeth = getRiotSpecial(idbuf)) ) {
+            yylval.riotMethod = riotMeth;
+            return RIOT_METHOD;
           } else if( (tiaMeth = getTiaSpecial(idbuf)) ) {
             yylval.tiaMethod = tiaMeth;
             return TIA_METHOD;
